from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``fr_FR`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_France
    """

    license_formats = (
        # New format
        "??-###-??",
        # Old format for plates < 2009
        "###-???-##",
    )



from .. import Provider as InternetProvider


class Provider(InternetProvider):

    free_email_domains = (
        "gmail.com",
        "hotmail.com",
        "yahoo.com",
        "outlook.com",
        "bih.net.ba",
        "tel.net.ba",
    )

    tlds = ("ba", "com.ba", "org.ba", "net.ba", "gov.ba", "edu.ba", "unsa.ba")

    replacements = (
        ("č", "c"),
        ("Č", "C"),
        ("ć", "c"),
        ("Ć", "C"),
        ("đ", "dj"),
        ("Đ", "Dj"),
        ("š", "s"),
        ("Š", "S"),
        ("ž", "z"),
        ("Ž", "Z"),
    )



import re
import sys
import time

import jmespath
from qrunner.core.api.request import HttpRequest, ResponseResult
from qrunner.core.android.driver import AndroidDriver
from qrunner.core.android.element import AndroidElement
from qrunner.core.ios.driver import IosDriver
from qrunner.core.ios.element import IosElement
from qrunner.core.browser.driver import BrowserDriver
from qrunner.core.browser.element import WebElement
from qrunner.utils.log import logger
from qrunner.utils.config import conf


class TestCase(HttpRequest):
    """
    测试用例基类，所有测试用例需要继承该类
    """

    # ---------------------初始化-------------------------------
    def start_class(self):
        """
        Hook method for setup_class fixture
        :return:
        """
        pass

    def end_class(self):
        """
        Hook method for teardown_class fixture
        :return:
        """
        pass

    @classmethod
    def setup_class(cls):
        # 从配置文件中获取浏览器相关配置（为了支持并发执行）
        platform = conf.get_name('common', 'platform')
        serial_no = conf.get_name('app', 'serial_no')
        pkg_name = conf.get_name('app', 'pkg_name')
        browser_name = conf.get_name('web', 'browser_name')

        # 初始化driver
        cls.driver = None
        if platform == 'android':
            logger.info('初始化 安卓 driver')
            if serial_no:
                cls.driver = AndroidDriver(serial_no, pkg_name)
            else:
                logger.info('serial_no为空')
                sys.exit()
        elif platform == 'ios':
            logger.info('初始化 IOS driver')
            if serial_no:
                cls.driver = IosDriver(serial_no, pkg_name)
            else:
                logger.info('serial_no为空')
                sys.exit()
        elif platform == 'browser':
            logger.info('初始化 Selenium driver')
            cls.driver = BrowserDriver(browser_name)
        elif platform == 'api':
            pass
        else:
            logger.info(f'不支持的平台: {platform}')
            sys.exit()
        cls().start_class()

    @classmethod
    def teardown_class(cls):
        # 关闭浏览器
        if isinstance(cls().driver, BrowserDriver):
            cls().driver.quit()
        cls().end_class()

    def start(self):
        """
        Hook method for setup_method fixture
        :return:
        """
        self.start_time = time.time()
        logger.debug(f"[start_time]: {time.strftime('%Y-%m-%d %H:%M:%S')}")

    def end(self):
        """
        Hook method for teardown_method fixture
        :return:
        """
        logger.debug(f"[end_time]: {time.strftime('%Y-%m-%d %H:%M:%S')}")
        take_time = time.time() - self.start_time
        logger.debug("[run_time]: {:.2f} s".format(take_time))

    def setup_method(self):
        # 启动应用
        if isinstance(self.driver, AndroidDriver) or isinstance(self.driver, IosDriver):
            self.driver.force_start_app()
        self.start()

    def teardown_method(self):
        self.end()
        # 退出应用
        if isinstance(self.driver, AndroidDriver) or isinstance(self.driver, IosDriver):
            self.driver.stop_app()

    # --------------------------------ui自动化相关封装------------------------------------

    def element(self, **kwargs):
        """
        定位元素
        :param kwargs: 元素定位方式
        :return: 根据平台返回对应的元素
        """
        if isinstance(self.driver, AndroidDriver):
            element = AndroidElement(self.driver, **kwargs)
        elif isinstance(self.driver, IosDriver):
            element = IosElement(self.driver, **kwargs)
        elif isinstance(self.driver, BrowserDriver):
            element = WebElement(self.driver, **kwargs)
        else:
            platform = conf.get('common', 'platform')
            logger.info(f'不支持的平台: {platform}，暂时只支持android、ios、browser')
            sys.exit()
        return element

    def click(self, **kwargs):
        """点击"""
        self.element(**kwargs).click()

    def click_exists(self, **kwargs):
        """存在才点击"""
        self.element(**kwargs).click_exists()

    def set_text(self, text, **kwargs):
        """输入"""
        self.element(**kwargs).set_text(text)

    def clear_text(self, **kwargs):
        """清除输入框"""
        self.element(**kwargs).clear_text()

    def get_text(self, **kwargs):
        """获取文本属性"""
        return self.element(**kwargs).text

    def assertText(self, expect_value):
        """断言页面包含文本"""
        page_source = self.driver.page_content
        assert expect_value in page_source, f'页面内容不包含 {expect_value}'

    def assertNotText(self, expect_value):
        """断言页面不包含文本"""
        page_source = self.driver.page_content
        assert expect_value not in page_source, f'页面内容不包含 {expect_value}'

    def assertElement(self, **kwargs):
        """断言元素存在"""
        assert self.element(**kwargs).exists(), f'元素 {kwargs} 不存在'

    def assertNotElement(self, **kwargs):
        """断言元素不存在"""
        assert not self.element(**kwargs).exists(), f'元素 {kwargs} 仍然存在'

    # -------------------仅web使用-----------------------
    def open_url(self, url, cookies=None):
        """打开页面"""
        self.driver.open_url(url)
        if cookies is not None:
            self.driver.add_cookies(cookies)
            self.driver.refresh()

    def assertTitle(self, expect_value):
        """断言页面标题等于"""
        title = self.driver.get_title()
        assert expect_value == title, f'页面标题 {title} 不等于 {expect_value}'

    def assertInTitle(self, expect_value):
        """断言页面标题包含"""
        title = self.driver.get_title()
        assert expect_value in title, f'页面标题 {title} 不包含 {expect_value}'

    def assertUrl(self, expect_value):
        """断言页面url等于"""
        url = self.driver.get_url()
        assert expect_value == url, f'页面url {url} 不等于 {expect_value}'

    def assertInUrl(self, expect_value):
        """断言页面url包含"""
        url = self.driver.get_url()
        assert expect_value in url, f'页面url {url} 不包含 {expect_value}'

    def assertAlertText(self, expect_value):
        """断言弹窗文本"""
        alert_text = self.driver.get_alert_text()
        assert expect_value == alert_text, f'弹窗文本 {alert_text} 等于 {expect_value}'

    # ---------------------------------接口自动化相关封装------------------------------------

    @staticmethod
    def assertStatusCode(status_code):
        """
        断言状态码
        """
        assert ResponseResult.status_code == status_code, \
            f'status_code {ResponseResult} != {status_code}'

    @staticmethod
    def assertPath(path, value):
        """
        断言响应json中对应path的值等于value
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert search_value == value, f'{search_value} != {value}'

    @staticmethod
    def assertEq(path, value):
        """
        assertPath换个名字
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert search_value == value, f'{search_value} != {value}'

    @staticmethod
    def assertLenEq(path, value):
        """
        断言列表长度等于多少
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert len(search_value) == value, f"{search_value} 的长度不等于 {value}"

    @staticmethod
    def assertLenGt(path, value):
        """
        断言列表长度大于多少
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert len(search_value) > value, f"{search_value} 的长度不大于 {value}"

    @staticmethod
    def assertLenGtOrEq(path, value):
        """
        断言列表长度大于等于多少
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert len(search_value) >= value, f"{search_value} 的长度不大于 {value}"

    @staticmethod
    def assertLenLt(path, value):
        """
        断言列表长度小于多少
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert len(search_value) < value, f"{search_value} 的长度不大于 {value}"

    @staticmethod
    def assertLenLtOrEq(path, value):
        """
        断言列表长度小于等于多少
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert len(search_value) <= value, f"{search_value} 的长度不大于 {value}"

    @staticmethod
    def assertGt(path, value):
        """
        值大于多少
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert int(search_value) > int(value), f"{search_value} 不大于 {value}"

    @staticmethod
    def assertGtOrEq(path, value):
        """
        值大于等于
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        if isinstance(search_value, str):
            search_value = int(search_value)
        assert search_value >= value, f"{search_value} 小于 {value}"

    @staticmethod
    def assertLt(path, value):
        """
        值小于多少
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert int(search_value) < int(value), f"{search_value} 不大于 {value}"

    @staticmethod
    def assertLtOrEq(path, value):
        """
        值小于等于多少
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert int(search_value) <= int(value), f"{search_value} 不大于 {value}"

    @staticmethod
    def assertNotEq(path, value):
        """
        值不等于
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert search_value != value, f"{search_value} 等于 {value}"

    @staticmethod
    def assertIn(path, value_list: list):
        """
        断言匹配结果被value_list包含
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert search_value in value_list, f"{value_list} 不包含 {search_value}"

    @staticmethod
    def assertNotExists(path):
        """断言字段不存在"""
        search_value = jmespath.search(path, ResponseResult.response)
        assert search_value is None, f'仍然包含 {path} 为 {search_value}'

    @staticmethod
    def assertContains(path, value):
        """
        断言匹配结果包含value
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        assert value in search_value, f"{search_value} 不包含 {value}"

    @staticmethod
    def assertTypeMatch(path, value_type):
        """
        类型匹配
        doc: https://jmespath.org/
        """
        if not isinstance(value_type, type):
            if value_type == 'int':
                value_type = int
            elif value_type == 'str':
                value_type = str
            elif value_type == 'list':
                value_type = list
            elif value_type == 'dict':
                value_type = dict
            else:
                value_type = str

        search_value = jmespath.search(path, ResponseResult.response)
        assert isinstance(search_value, value_type), f'{search_value} 不是 {value_type} 类型'

    @staticmethod
    def assertStartsWith(path, value):
        """
        以什么开头
        doc: https://jmespath.org/
        """
        search_value: str = jmespath.search(path, ResponseResult.response)
        assert search_value.startswith(value), f'{search_value} 不以 {value} 开头'

    @staticmethod
    def assertEndsWith(path, value):
        """
        以什么结尾
        doc: https://jmespath.org/
        """
        search_value: str = jmespath.search(path, ResponseResult.response)
        assert search_value.endswith(value), f'{search_value} 不以 {value} 结尾'

    @staticmethod
    def assertRegexMatch(path, value):
        """
        正则匹配
        doc: https://jmespath.org/
        """
        search_value = jmespath.search(path, ResponseResult.response)
        match_obj = re.match(r'' + value, search_value, flags=re.I)
        assert match_obj is not None, f'结果 {search_value} 匹配失败'


class Page:
    """
    测试页面基类，所有页面需要继承该类
    """
    def __init__(self, driver):
        """
        :param driver: 驱动句柄
        """
        self.driver = driver

    def element(self, **kwargs):
        """
        根据
        :param kwargs: 元素定位方式
        :return: 根据平台返回对应的元素
        """
        if isinstance(self.driver, AndroidDriver):
            element = AndroidElement(self.driver, **kwargs)
        elif isinstance(self.driver, IosDriver):
            element = IosElement(self.driver, **kwargs)
        elif isinstance(self.driver, BrowserDriver):
            element = WebElement(self.driver, **kwargs)
        else:
            platform = conf.get('common', 'platform')
            logger.info(f'不支持的平台: {platform}，暂时只支持android、ios、browser')
            sys.exit()
        return element




from .. import Provider as CurrencyProvider


class Provider(CurrencyProvider):

    price_formats = ["#,##", "%#,##", "%##,##", "%.###,##", "%#.###,##"]

    def pricetag(self):
        return self.numerify(self.random_element(self.price_formats)) + " kr."



from collections import OrderedDict

from .. import Provider as ColorProvider

localized = True


class Provider(ColorProvider):
    """Implement color provider for ``th_TH`` locale.

    Sources:
    - https://th.wikipedia.org/wiki/รายชื่อสี
    """

    all_colors = OrderedDict(
        (
            ("สีดำ", "#000000"),
            ("สีน้ำเงินเขียว", "#0095B6"),
            ("สีน้ำเงินม่วง", "#8A2BE2"),
            ("สีทองแดง", "#CD7F32"),
            ("สีน้ำตาล", "#964B00"),
            ("สีกาแฟ", "#6F4E37"),
            ("สีทอง", "#FFD700"),
            ("สีเทา", "#808080"),
            ("สีเขียว", "#00FF00"),
            ("สีหยก", "#00A86B"),
            ("สีส้ม", "#FFA500"),
            ("สีส้มแดง", "#FF4500"),
            ("สีออร์คิด", "#DA70D6"),
            ("สีชมพู", "#FFC0CB"),
            ("สีม่วง", "#800080"),
            ("สีแดง", "#FF0000"),
            ("สีเงิน", "#C0C0C0"),
            ("สีขาว", "#FFFFFF"),
            ("สีเหลือง", "#FFFF00"),
        )
    )

    safe_colors = (
        "สีดำ",
        "สีน้ำตาล",
        "สีทอง",
        "สีเขียว",
        "สีส้ม",
        "สีชมพู",
        "สีม่วง",
        "สีเงิน",
        "สีแดง",
        "สีเงิน",
        "สีขาว",
        "สีเหลือง",
    )



import binascii
import os
import subprocess
import time
import allure
import requests
import wda
from qrunner.utils.log import logger
from qrunner.utils.config import conf

# # 定义最大重连次数
# max_retry = 3
#
#
# def launch_wda(serial_no):
#     logger.info(f'{serial_no} 开始启动wda服务')
#     cmd = 'tidevice -u {0} xctest'.format(serial_no)
#     subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
#
#
# # 杀掉tidevice进程
# def stop_wda():
#     logger.info('\n停止wda服务')
#     try:
#         if os.name == 'nt':
#             os.popen('taskkill /f /im tidevice.exe')
#         else:
#             os.popen("ps -ef | grep tidevice | awk '{print $2}' | xargs kill -9")
#     except Exception as e:
#         logger.error(f'停止wda服务失败: {str(e)}')
#     else:
#         logger.info('停止wda服务成功')
#
#
# def relaunch_wda(func):
#     def wrapper(self, *args, **kwargs):
#         return _inner_retry(self, func, *args, **kwargs)
#     return wrapper
#
#
# def _inner_retry(self, func, *args, **kwargs):
#     global max_retry
#
#     while max_retry > 0:
#         try:
#             return func(self, *args, **kwargs)
#         except (requests.exceptions.ConnectionError, wda.WDAError,
#                 binascii.Error) as _:
#             time.sleep(3)
#             logger.info(f'开始重连wda')
#             # serial_no = conf.get_name('app', 'serial_no')
#             launch_wda(self.serial_no)
#             max_retry -= 1
#             logger.info(f'剩余重连次数: {max_retry}')
#     return func(self, *args, **kwargs)


class IosDriver(object):
    # _instance = {}
    #
    # def __new__(cls, serial_no=None, bundle_id=None):
    #     if not serial_no:
    #         serial_no = conf.get_name('app', 'serial_no')
    #     if serial_no not in cls._instance:
    #         cls._instance[serial_no] = super().__new__(cls)
    #     return cls._instance[serial_no]

    def __init__(self, serial_no=None, pkg_name=None):
        if not serial_no:
            serial_no = conf.get_name('app', 'serial_no')
        self.serial_no = serial_no
        if not pkg_name:
            pkg_name = conf.get_name('app', 'pkg_name')
        self.bundle_id = pkg_name
        logger.info(self.bundle_id)
        logger.info(f'启动 ios driver for {self.serial_no}')
        self.d = wda.USBClient(self.serial_no, port=8100)

    # @classmethod
    # def get_instance(cls, serial_no=None):
    #     if serial_no not in cls._instance:
    #         logger.info(f'{serial_no} Create ios driver singleton')
    #         return IosDriver(serial_no)
    #     return IosDriver._instance[serial_no]
    #
    # @classmethod
    # def get_remote_instance(cls, server_url, token):
    #     device = Device(server_url, token)
    #     d = wda.Client(device.get_device(platform='apple'))
    #     return d, device

    def install_app(self, ipa_url):
        cmd = f'tidevice -u {self.serial_no} install {ipa_url}'
        logger.info(f'安装应用: {ipa_url}')
        output = subprocess.getoutput(cmd)
        if 'Complete' in output.split()[-1]:
            logger.info(f'{self.serial_no} 安装应用{ipa_url} 成功')
            return
        else:
            logger.info(f'{self.serial_no} 安装应用{ipa_url}失败，因为{output}')

    def new_install_app(self, ipa_url):
        self.uninstall_app(self.bundle_id)
        cmd = f'tidevice -u {self.serial_no} install {ipa_url}'
        logger.info(f'安装应用: {ipa_url}')
        output = subprocess.getoutput(cmd)
        if 'Complete' in output.split()[-1]:
            logger.info(f'{self.serial_no} 安装应用{ipa_url} 成功')
            return
        else:
            logger.info(f'{self.serial_no} 安装应用{ipa_url}失败，因为{output}')

    def uninstall_app(self, bundle_id=None):
        if not bundle_id:
            bundle_id = self.bundle_id
        cmd = f'tidevice -u {self.serial_no} uninstall {bundle_id}'
        logger.info(f'卸载应用: {bundle_id}')
        output = subprocess.getoutput(cmd)
        if 'Complete' in output.split()[-1]:
            logger.info(f'{self.serial_no} 卸载应用{bundle_id} 成功')
            return
        else:
            logger.info(f'{self.serial_no} 卸载应用{bundle_id}失败，因为{output}')

    def start_app(self, bundle_id=None):
        if not bundle_id:
            bundle_id = self.bundle_id
        logger.info(f'启动应用: {bundle_id}')
        self.d.app_start(bundle_id)

    def force_start_app(self, bundle_id=None):
        if not bundle_id:
            bundle_id = self.bundle_id
        logger.info(f'强制启动应用: {bundle_id}')
        self.d.app_terminate(bundle_id)
        self.start_app(bundle_id)

    def stop_app(self, bundle_id=None):
        if not bundle_id:
            bundle_id = self.bundle_id
        logger.info(f'停止应用: {bundle_id}')
        self.d.app_terminate(bundle_id)

    def app_current(self):
        cur_apps = self.d.app_current()
        logger.info(f'获取运行中的app列表: {cur_apps}')
        return cur_apps

    def app_launch(self, bundle_id=None):
        if not bundle_id:
            bundle_id = self.bundle_id
        logger.info(f'将应用切到前台: {bundle_id}')
        self.d.app_launch(bundle_id)

    def back(self):
        logger.info('返回上一页')
        time.sleep(1)
        self.d.swipe(0, 100, 100, 100)

    def go_home(self):
        logger.info('返回手机主页')
        self.d.home()

    def send_keys(self, value):
        logger.info(f'输入: {value}')
        self.d.send_keys(value)

    def screenshot(self, file_name):
        """
        截图并保存到预定路径并上传到allure报告
        @param file_name: foo.png or fool
        @return:
        """
        # 把文件名处理成test.png的样式
        if '.' in file_name:
            file_name = file_name.split(r'.')[0]
        # 截图并保存到当前目录的images文件夹中
        img_dir = os.path.join(os.getcwd(), 'images')
        if os.path.exists(img_dir) is False:
            os.mkdir(img_dir)
        time_str = time.strftime('%Y%m%d%H%M%S')
        file_path = os.path.join(img_dir,
                                 f'{file_name}-{time_str}.png')
        logger.info(f'截图保存至: {file_path}')
        self.d.screenshot(file_path)
        # 截图并上传到allure报告
        logger.info(f'截图上传至allure报告')
        allure.attach.file(file_path,
                           attachment_type=allure.attachment_type.PNG,
                           name=f'{file_name}-{time_str}')

    # def upload_pic(self, filename):
    #     self.screenshot('tmp.png')
    #     allure.attach.file('tmp.png',
    #                        attachment_type=allure.attachment_type.PNG,
    #                        name=f'{filename}-{get_time()}')
    #     os.remove('tmp.png')

    @property
    def page_content(self):
        page_source = self.d.source(accessible=False)
        logger.info(f'获取页面内容: \n{page_source}')
        return page_source

    def get_window_size(self):
        size = self.d.window_size()
        logger.info(f'获取屏幕尺寸: {size}')
        return size

    def click(self, x, y):
        logger.info(f'点击坐标: ({x}, {y})')
        logger.info(f'{self.serial_no} Tap point ({x}, {y})')
        self.d.appium_settings({"snapshotMaxDepth": 0})
        self.d.tap(x, y)
        self.d.appium_settings({"snapshotMaxDepth": 50})
        time.sleep(1)

    def double_click(self, x, y):
        logger.info(f'双击坐标: {x},{y}')
        self.d.double_tap(x, y)

    def tap_hold(self, x, y):
        logger.info(f'长按: {x},{y}')
        self.d.tap_hold(x, y, 1.0)

    def click_alert(self, alert_list: list):
        try:
            self.d.alert.click(alert_list)
        except:
            pass

    def swipe(self, start_x, start_y, end_x, end_y, duration=0):
        logger.info(f'从坐标({start_x}, {start_y})滑动到({end_x}, {end_y})')
        logger.info(f'{self.serial_no} swipe from point ({start_x}, {start_y}) to ({end_x}, {end_y})')
        self.d.appium_settings({"snapshotMaxDepth": 2})
        self.d.swipe(int(start_x), int(start_y), int(end_x), int(end_y), duration)
        self.d.appium_settings({"snapshotMaxDepth": 50})
        time.sleep(2)

    def swipe_by_screen_percent(self, start_x_percent, start_y_percent, end_x_percent, end_y_percent, duration=0):
        logger.info(f'根据屏幕百分比进行滑动')
        w, h = self.d.window_size()
        start_x = w * start_x_percent
        start_y = h * start_y_percent
        end_x = w * end_x_percent
        end_y = h * end_y_percent
        self.swipe(start_x, start_y, end_x, end_y, duration=duration)

    def swipe_left(self, start_percent=1, end_percent=0.5):
        logger.info('往左边滑动')
        w, h = self.d.window_size()
        self.swipe(start_percent * (w - 1), h/2, end_percent * w, h/2)

    def swipe_right(self, start_percent=0.5, end_percent=1):
        logger.info('往右边滑动')
        w, h = self.d.window_size()
        self.swipe(start_percent * w, h / 2, end_percent * (w - 1), h / 2)

    def swipe_up(self, start_percent=0.8, end_percent=0.2):
        logger.info('往上边滑动')
        w, h = self.d.window_size()
        self.swipe(w / 2, start_percent * h, w / 2, end_percent * h)

    def swipe_down(self, start_percent=0.2, end_percent=0.8):
        logger.info('往下面滑动')
        w, h = self.d.window_size()
        self.swipe(w / 2, start_percent * h, w / 2, end_percent * h)

    def check(self):
        logger.info('健康检查')
        self.d.healthcheck()

    def locked(self):
        logger.info('是否锁屏')
        status = self.d.locked()
        logger.info(status)
        return status

    def lock(self):
        logger.info('锁住屏幕')
        self.d.lock()

    def unlock(self):
        logger.info('解锁屏幕')
        self.d.unlock()

    def open_url(self, url):
        """
        打开schema
        @param: url，schema链接，taobao://m.taobao.com/index.htm
        @return:
        """
        logger.info(f'打开url: {url}')
        self.d.open_url(url)

    @property
    def battery_info(self):
        info = self.d.battery_info()
        logger.info(f'电池信息: {info}')
        return info

    @property
    def device_info(self):
        info = self.d.device_info()
        logger.info(f'设备信息: {info}')
        return info

    @property
    def scale(self):
        scale = self.d.scale
        logger.info(f'设备分辨率: {scale}')
        return scale









# @Time    : 2022/2/22 9:35
# @Author  : kang.yang@qizhidao.com
# @File    : request.py
import sys
from urllib import parse
import requests
import json as json_util
from qrunner.utils.config import conf
from qrunner.utils.log import logger

IMG = ["jpg", "jpeg", "gif", "bmp", "webp"]


def request(func):
    def wrapper(*args, **kwargs):
        func_name = func.__name__
        print("\n")
        logger.info('-------------- Request -----------------[🚀]')
        try:
            url = list(args)[1]
        except IndexError:
            url = kwargs.get("url", "")

        if "http" not in url:
            base_url = conf.get_name('common', 'base_url')
            if 'http' in base_url:
                url = parse.urljoin(base_url, url)
            else:
                logger.debug('请设置正确的base_url')
                sys.exit()

        img_file = False
        file_type = url.split(".")[-1]
        if file_type in IMG:
            img_file = True

        logger.debug("[method]: {m}      [url]: {u} \n".format(m=func_name.upper(), u=url))
        auth = kwargs.get("auth", "")

        # 处理请求头
        headers: dict = json_util.loads(conf.get_name('api', 'headers'))  # 从配置文件拿到登录用户请求头
        # 如果login=False，从请求头中删掉登录态相关的键值对
        login_status = kwargs.get('login', True)
        if not login_status:
            login_key = (conf.get_name('api', 'login_key')).split(',')
            for key in login_key:
                headers.pop(key)
        # 把上层请求方法的headers参数更新到headers里面
        headers.update(kwargs.pop("headers", {}))
        kwargs['headers'] = headers

        # 设置请求超时时间为5s
        try:
            timeout_default = int(conf.get_name('common', 'timeout'))
        except:
            timeout_default = 10
        timeout_set = kwargs.pop("timeout", None)
        if timeout_set is not None:
            kwargs['timeout'] = timeout_set
        else:
            kwargs['timeout'] = timeout_default

        cookies = kwargs.get("cookies", "")
        params = kwargs.get("params", "")
        data = kwargs.get("data", "")
        json = kwargs.get("json", "")
        if auth != "":
            logger.debug(f"[auth]:\n {json_util.dumps(auth)} \n")
        if headers != "":
            # logger.debug(type(headers))
            logger.debug(f"[headers]:\n {json_util.dumps(headers)} \n")
        if cookies != "":
            logger.debug(f"[cookies]:\n {json_util.dumps(cookies)} \n")
        if params != "":
            logger.debug(f"[params]:\n {json_util.dumps(params)} \n")
        if data != "":
            logger.debug(f"[data]:\n {json_util.dumps(data)} \n")
        if json != "":
            logger.debug(f"[json]:\n {json_util.dumps(json)} \n")

        # running function
        r = func(*args, **kwargs)

        ResponseResult.status_code = r.status_code
        logger.info("-------------- Response ----------------")
        try:
            resp = r.json()
            logger.debug(f"[type]: json \n")
            logger.debug(f"[response]:\n {resp} \n")
            ResponseResult.response = resp
        except BaseException as msg:
            logger.debug("[warning]: {} \n".format(msg))
            if img_file is True:
                logger.debug("[type]: {}".format(file_type))
                ResponseResult.response = r.content
            else:
                logger.debug("[type]: text \n")
                logger.debug(f"[response]:\n {r.text} \n")
                ResponseResult.response = r.text

    return wrapper


class ResponseResult:
    status_code = 200
    response = None


class HttpRequest(object):

    @request
    def get(self, url, params=None, login=True, **kwargs):
        if "http" not in url:
            base_url = eval(conf.get_name('common', 'base_url'))
            if 'http' in base_url:
                url = parse.urljoin(base_url, url)
            else:
                logger.debug('请设置正确的base_url')
                sys.exit()
        return requests.get(url, params=params, **kwargs)

    @request
    def post(self, url, data=None, json=None, login=True, **kwargs):
        if "http" not in url:
            base_url = conf.get_name('common', 'base_url')
            logger.debug(base_url)
            if 'http' in base_url:
                url = parse.urljoin(base_url, url)
            else:
                logger.debug('请设置正确的base_url')
                sys.exit()
        return requests.post(url, data=data, json=json, **kwargs)

    @request
    def put(self, url, data=None, json=None, login=True, **kwargs):
        if "http" not in url:
            base_url = conf.get_name('common', 'base_url')
            if 'http' in base_url:
                url = parse.urljoin(base_url, url)
            else:
                logger.debug('请设置正确的base_url')
                sys.exit()
        if json is not None:
            data = json_util.dumps(json)
        return requests.put(url, data=data, **kwargs)

    @request
    def delete(self, url, login=True, **kwargs):
        if "http" not in url:
            base_url = conf.get_name('common', 'base_url')
            if 'http' in base_url:
                url = parse.urljoin(base_url, url)
            else:
                logger.debug('请设置正确的base_url')
                sys.exit()
        return requests.delete(url, **kwargs)

    @property
    def response(self):
        """
        Returns the result of the response
        :return: response
        """
        return ResponseResult.response

    @property
    def session(self):
        """
        A Requests session.
        """
        s = requests.Session()
        return s

    @staticmethod
    def request(method=None, url=None, headers=None, files=None, data=None,
                params=None, auth=None, cookies=None, hooks=None, json=None):
        """
        A user-created :class:`Request <Request>` object.
        """
        req = requests.Request(method, url, headers, files, data,
                               params, auth, cookies, hooks, json)
        return req




from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}}-{{last_name}}",
        "{{last_name}}-{{last_name}} {{company_suffix}}",
        "{{last_name}}, {{last_name}} και {{last_name}}",
    )
    company_suffixes = ("Α.Ε.", "και υιοί", "Ο.Ε.", "Α.Β.Ε.Ε.", "Α.Ε. ΟΜΙΛΟΣ ΕΤΑΙΡΕΙΩΝ")



# coding=utf-8
# @Time    : 2022/2/21 11:21
# @Author  : kang.yang@qizhidao.com
# @File    : mongo_util.py
import pymongo


class BaseDB(object):
    def __init__(self, ip, db_name, table_name):
        self.client = pymongo.MongoClient(f"mongodb://{ip}:27017/")
        self.db = self.client[db_name]
        self.table = self.db[table_name]

    def insert_one(self, data: dict):
        print(f'插入数据: {data}')
        try:
            result = self.table.insert_one(data)
        except Exception as e:
            print(e)
        else:
            print('插入成功')
            print(result.inserted_id)
        finally:
            self.client.close()

    def insert_list(self, data: list):
        print(f'插入数据: {data}')
        try:
            result = self.table.insert_many(data)
        except Exception as e:
            print(e)
        else:
            print('插入成功')
            print(result.inserted_ids)
        finally:
            self.client.close()

    def select_all(self):
        print('查询表中所有数据')
        try:
            result = list(self.table.find())
        except Exception as e:
            print(e)
        else:
            print('查询成功')
            # print(result)
            return result
        finally:
            self.client.close()

    def select(self, query_data: dict):
        print(f'查询满足 {query_data} 条件的数据')
        try:
            result = list(self.table.find(query_data))
        except Exception as e:
            print(e)
        else:
            print('查询成功')
            # print(result)
            return result
        finally:
            self.client.close()

    def delete_all(self):
        print(f'删除所有数据')
        try:
            result = self.table.delete_many({})
        except Exception as e:
            print(e)
        else:
            print(result.deleted_count, "个文档已删除")
        finally:
            self.client.close()







from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``ru_RU`` locale."""

    word_list = (
        "войти",
        "монета",
        "вскинуть",
        "желание",
        "экзамен",
        "налоговый",
        "вытаскивать",
        "приятель",
        "вздрагивать",
        "куча",
        "порт",
        "точно",
        "заплакать",
        "изба",
        "правление",
        "художественный",
        "мучительно",
        "изображать",
        "фонарик",
        "миф",
        "грустный",
        "опасность",
        "мера",
        "пастух",
        "факультет",
        "мелькнуть",
        "полевой",
        "другой",
        "выраженный",
        "забирать",
        "рот",
        "народ",
        "соответствие",
        "тута",
        "коммунизм",
        "решение",
        "плод",
        "собеседник",
        "возмутиться",
        "достоинство",
        "господь",
        "болото",
        "инфекция",
        "голубчик",
        "сынок",
        "пространство",
        "прощение",
        "прежде",
        "хотеть",
        "ленинград",
        "даль",
        "развитый",
        "близко",
        "более",
        "спорт",
        "эпоха",
        "ответить",
        "освободить",
        "совет",
        "проход",
        "палец",
        "вчера",
        "приличный",
        "ярко",
        "белье",
        "кузнец",
        "неожиданно",
        "вперед",
        "зато",
        "кольцо",
        "передо",
        "мгновение",
        "плавно",
        "табак",
        "число",
        "изучить",
        "тяжелый",
        "рассуждение",
        "салон",
        "идея",
        "что",
        "светило",
        "порода",
        "сомнительный",
        "бок",
        "очко",
        "неудобно",
        "советовать",
        "отдел",
        "помолчать",
        "поздравлять",
        "пробовать",
        "дошлый",
        "смеяться",
        "упорно",
        "вздрогнуть",
        "затянуться",
        "танцевать",
        "песенка",
        "выбирать",
        "правильный",
        "намерение",
        "издали",
        "запустить",
        "наслаждение",
        "крыса",
        "лететь",
        "космос",
        "радость",
        "поезд",
        "находить",
        "гулять",
        "горький",
        "бочок",
        "ночь",
        "счастье",
        "уничтожение",
        "дьявол",
        "коробка",
        "спасть",
        "кожа",
        "провинция",
        "прелесть",
        "тюрьма",
        "низкий",
        "сверкать",
        "темнеть",
        "солнце",
        "дружно",
        "настать",
        "блин",
        "степь",
        "самостоятельно",
        "крутой",
        "картинка",
        "зачем",
        "рабочий",
        "необычный",
        "армейский",
        "труп",
        "ягода",
        "около",
        "монета",
        "естественный",
        "юный",
        "район",
        "скрытый",
        "поймать",
        "строительство",
        "палата",
        "миг",
        "триста",
        "штаб",
        "ломать",
        "возможно",
        "полюбить",
        "человечек",
        "легко",
        "чувство",
        "ручей",
        "карман",
        "деньги",
        "неправда",
        "сравнение",
        "грудь",
        "отъезд",
        "возникновение",
        "степь",
        "возбуждение",
        "деловой",
        "следовательно",
        "жидкий",
        "сынок",
        "художественный",
        "поколение",
        "расстегнуть",
        "пища",
        "ученый",
        "секунда",
        "успокоиться",
        "вряд",
        "аж",
        "вскакивать",
        "мимо",
        "падать",
        "потянуться",
        "угроза",
        "растеряться",
        "бегать",
        "стакан",
        "о",
        "кпсс",
        "ныне",
        "пол",
        "реклама",
        "при",
        "школьный",
        "премьера",
        "дальний",
        "потрясти",
        "освобождение",
        "покидать",
        "наступать",
        "жить",
        "какой",
        "обида",
        "командование",
        "девка",
        "выражаться",
        "головной",
        "второй",
        "князь",
        "социалистический",
        "головка",
        "привлекать",
        "через",
        "господь",
        "результат",
        "отметить",
        "ведь",
        "падаль",
        "покидать",
        "художественный",
        "правый",
        "висеть",
        "лапа",
        "каюта",
        "слишком",
        "нервно",
        "серьезный",
        "зима",
        "заработать",
        "эффект",
        "пропасть",
        "плод",
        "что",
        "висеть",
        "холодно",
        "единый",
        "выкинуть",
        "мрачно",
        "выгнать",
        "умирать",
        "иной",
        "космос",
        "природа",
        "функция",
        "поставить",
        "оборот",
        "услать",
        "очередной",
        "медицина",
        "функция",
        "зарплата",
        "выдержать",
        "расстройство",
        "адвокат",
        "задержать",
        "появление",
        "инвалид",
        "интеллектуальный",
        "исследование",
        "господь",
        "смертельный",
        "спичка",
        "вариант",
        "рай",
        "одиннадцать",
        "чем",
        "манера",
        "магазин",
        "поговорить",
        "полоска",
        "помимо",
        "построить",
        "домашний",
        "механический",
        "сохранять",
        "отражение",
        "научить",
        "тесно",
        "аллея",
        "прежний",
        "посидеть",
        "славный",
        "очутиться",
        "лететь",
        "невозможно",
        "порядок",
        "выразить",
        "спешить",
        "сынок",
        "ребятишки",
        "угроза",
        "оставить",
        "цвет",
        "налево",
        "парень",
        "миллиард",
        "горький",
        "трубка",
        "подробность",
        "пасть",
        "непривычный",
        "угодный",
        "засунуть",
        "цель",
        "запретить",
        "дремать",
        "разуметься",
        "приходить",
        "совещание",
        "постоянный",
        "анализ",
        "терапия",
        "приятель",
        "процесс",
        "академик",
        "металл",
        "развернуться",
        "жестокий",
        "интернет",
        "банда",
        "изменение",
        "коллектив",
        "похороны",
        "устройство",
        "торопливый",
        "разводить",
        "промолчать",
        "подземный",
        "пламя",
        "редактор",
        "теория",
        "карандаш",
        "упор",
        "означать",
        "бабочка",
        "четыре",
        "столетие",
        "разнообразный",
        "витрина",
        "нож",
        "команда",
        "шлем",
        "недостаток",
        "протягивать",
        "за",
        "металл",
        "добиться",
        "сутки",
        "четко",
        "предоставить",
        "тысяча",
        "запеть",
        "бригада",
        "мелочь",
        "выраженный",
        "пересечь",
        "сходить",
        "вообще",
        "рис",
        "банк",
        "бак",
        "передо",
        "назначить",
        "важный",
        "правление",
        "палка",
        "трясти",
        "уронить",
        "витрина",
        "основание",
        "да",
        "перебивать",
        "дыхание",
        "применяться",
        "июнь",
        "бетонный",
        "избегать",
        "умолять",
        "мягкий",
        "заявление",
        "конференция",
        "встать",
        "свежий",
        "сопровождаться",
        "цепочка",
        "выражение",
        "угол",
        "ботинок",
        "ложиться",
        "инструкция",
        "присесть",
        "решетка",
        "еврейский",
        "порог",
        "зеленый",
        "граница",
        "ставить",
        "смелый",
        "сустав",
        "роса",
        "демократия",
        "вывести",
        "конструкция",
        "задрать",
        "багровый",
        "военный",
        "направо",
        "житель",
        "товар",
        "неправда",
        "материя",
        "командующий",
        "кидать",
        "заложить",
        "лиловый",
        "слать",
        "горький",
        "пространство",
        "провал",
        "мусор",
        "наткнуться",
        "торговля",
        "монета",
        "место",
        "спалить",
        "бровь",
        "левый",
        "хлеб",
        "коричневый",
        "потом",
        "страсть",
        "виднеться",
        "роскошный",
        "способ",
        "костер",
        "заведение",
        "пропадать",
        "слишком",
        "пятеро",
        "мальчишка",
        "тусклый",
        "неожиданный",
        "плясать",
        "дурацкий",
        "дрогнуть",
        "сбросить",
        "прошептать",
        "беспомощный",
        "рота",
        "песня",
        "тревога",
        "некоторый",
        "термин",
        "нажать",
        "видимо",
        "валюта",
        "набор",
        "боец",
        "райком",
        "новый",
        "скользить",
        "руководитель",
        "волк",
        "изредка",
        "понятный",
        "пропаганда",
        "остановить",
        "исполнять",
        "ход",
        "госпожа",
        "печатать",
        "командир",
        "снимать",
        "казнь",
        "невыносимый",
        "спорт",
        "тревога",
        "уточнить",
        "актриса",
        "полностью",
        "покинуть",
        "сверкающий",
        "мотоцикл",
        "дорогой",
        "указанный",
        "ремень",
        "посвятить",
        "один",
        "а",
        "доставать",
        "хозяйка",
        "носок",
        "написать",
        "еврейский",
        "призыв",
        "увеличиваться",
        "равнодушный",
    )



from typing import Tuple

from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats_female: Tuple[str, ...] = (
        "{{first_name_female}} {{last_name}}",
        "{{prefix_female}} {{first_name_female}} {{last_name}}",
    )

    formats_male: Tuple[str, ...] = (
        "{{first_name_male}} {{last_name}}",
        "{{prefix_male}} {{first_name_male}} {{last_name}}",
    )

    formats = formats_male + formats_female

    first_names_female: Tuple[str, ...] = (
        "آلاء",
        "آيات",
        "أجوان",
        "أحلام",
        "أروى",
        "أريج",
        "أزهار",
        "أسرار",
        "أسيل",
        "أغاريد",
        "أفراح",
        "أفنان",
        "ألين",
        "أناهيد",
        "إباء",
        "إخلاص",
        "إلينا",
        "ابتسام",
        "ابتكار",
        "ابتهاج",
        "ابتهال",
        "اصيل",
        "اعتكاف",
        "اعتماد",
        "افتكار",
        "ايمان",
        "بارعة",
        "باسمة",
        "باهرة",
        "بتلاء",
        "بتول",
        "بثينة",
        "بدرالدّجى",
        "بشرى",
        "بلسم",
        "بلقيس",
        "بلماء",
        "بلند",
        "بنان",
        "بنفسج",
        "بهاء",
        "بهجة",
        "بهية",
        "بوران",
        "بيسان",
        "بيلسان",
        "تالا",
        "تاليا",
        "ترانيم",
        "ترف",
        "تمام",
        "تولين",
        "جالا",
        "جلنار",
        "جمان",
        "جميلة",
        "جنى",
        "جهراء",
        "جوان",
        "جوانا",
        "جواهر",
        "جود",
        "جودي",
        "جوريّة",
        "جوليا",
        "جوين",
        "جيلان",
        "حلا",
        "حياة",
        "خاشعة",
        "دارين",
        "دانة",
        "دانية",
        "دعاء",
        "ديمه",
        "راما",
        "ربى",
        "رواء",
        "روبين",
        "روعة",
        "روفيدا",
        "ريان",
        "ريتاج",
        "ريتال",
        "ريف",
        "ريفال",
        "ريم",
        "ريما",
        "ريمان",
        "ريناد",
        "زكية",
        "زهرة",
        "سبأ",
        "سجى",
        "سديم",
        "سلاف",
        "سلسبيل",
        "شادن",
        "شهد",
        "ضحى",
        "ضياء",
        "عالية",
        "عتاب",
        "غوى",
        "غيداء",
        "فداء",
        "فرات",
        "فردوس",
        "كاملة",
        "كرمة",
        "كوثر",
        "لارا",
        "لاما",
        "لتين",
        "لوجين",
        "لورا",
        "لورين",
        "لوليا",
        "ليان",
        "ليساء",
        "ليم",
        "لينا",
        "مادلين",
        "ماذى",
        "مايا",
        "مريم",
        "ميار",
        "ميرا",
        "ميرال",
        "ميسون",
        "ميلاء",
        "ناديه",
        "ناردين",
        "ناهد",
        "نشوة",
        "نغم",
        "نوال",
        "نوره",
        "نوف",
        "هاجر",
        "هايدي",
        "هدى",
        "هناء",
        "هنادي",
        "هند",
        "هيا",
        "هيام",
        "سجا",
        "وصاف",
        "وفاء",
        "يارا",
        "ياسمين",
        "يسرى",
    )

    first_names_male: Tuple[str, ...] = (
        "أحمد",
        "تاج",
        "تاج الدّين",
        "تامر",
        "تحسين",
        "ترف",
        "تقي",
        "تقيّ الدّين",
        "تميم",
        "تمّام",
        "توفيق",
        "ثائر",
        "ثابت",
        "ثاقب",
        "ثامر",
        "ثروت",
        "ثقيف",
        "جابر",
        "جاد",
        "جاسم",
        "جدير",
        "جرير",
        "جرّاح",
        "جسور",
        "جعفر",
        "جلاء",
        "جلال",
        "جلال الدّين",
        "جليل",
        "جمال",
        "جمال الدّين",
        "جميل",
        "جهاد",
        "حاتم",
        "حارث",
        "حازم",
        "حافظ",
        "حامد",
        "حبّاب",
        "حسام",
        "حسن",
        "حسني",
        "حسنين",
        "حسيب",
        "حسين",
        "حفيظ",
        "حقّي",
        "حكيم",
        "حليم",
        "حمدان",
        "حمدي",
        "حمزة",
        "حمود",
        "حميد",
        "حمّاد",
        "حنبل",
        "حنفي",
        "حيدر",
        "حيّان",
        "خاطر",
        "خافق",
        "خالد",
        "خالدي",
        "خضر",
        "خطيب",
        "خلدون",
        "خلف",
        "خلوصي",
        "خليفة",
        "خليل",
        "خميس",
        "خيري",
        "دؤوب",
        "داني",
        "داهي",
        "داوود",
        "دريد",
        "دليل",
        "دهمان",
        "ديسم",
        "ذريع",
        "ذكي",
        "ذيب",
        "رؤوف",
        "رئيس",
        "رائد",
        "رائف",
        "رابح",
        "راتب",
        "راجح",
        "راجي",
        "رازي",
        "راسم",
        "راشد",
        "راضي",
        "راغب",
        "رافع",
        "رامح",
        "رامز",
        "رامي",
        "راني",
        "راوي",
        "رباح",
        "ربيع",
        "رجاء",
        "رجائي",
        "رجب",
        "رحيب",
        "رخاء",
        "رزين",
        "رستم",
        "رسمي",
        "رشاد",
        "رشدي",
        "رشيد",
        "رضوان",
        "رضي",
        "رفيق",
        "رمحي",
        "رمزي",
        "رمضان",
        "رهيف",
        "روحي",
        "ريّان",
        "زاخر",
        "زاكي",
        "زاهر",
        "زاهي",
        "زايد",
        "زبير",
        "زغلول",
        "زكريا",
        "زكي",
        "زهدي",
        "زهران",
        "زهير",
        "زياد",
        "زيد",
        "زيدان",
        "زين",
        "سائد",
        "ساجد",
        "ساجي",
        "ساطع",
        "سالم",
        "سامح",
        "سامر",
        "سامي",
        "ساهد",
        "ساهر",
        "سخاء",
        "سراج",
        "سراج الدّين",
        "سرحان",
        "سرور",
        "سعد",
        "سعدون",
        "سعدي",
        "سعود",
        "سعيد",
        "سفيان",
        "سفير",
        "سلام",
        "سلطان",
        "سلمان",
        "سليم",
        "سليمان",
        "سموح",
        "سمير",
        "سنام",
        "سنان",
        "سهل",
        "سهوان",
        "سهيل",
        "سيف الدّين",
        "سيّد",
        "شادي",
        "شاطر",
        "شافع",
        "شاكر",
        "شامخ",
        "شامل",
        "شبلي",
        "شبيب",
        "شجاع",
        "شدّاد",
        "شريف",
        "شعبان",
        "شعلان",
        "شعيب",
        "شفيع",
        "شكري",
        "شكيب",
        "شهاب",
        "شهب",
        "شهم",
        "شهير",
        "شوقي",
        "شيّق",
        "صائب",
        "صابر",
        "صاحب",
        "صادح",
        "صادق",
        "صارم",
        "صافي",
        "صالح",
        "صامد",
        "صباح",
        "صبحي",
        "صبري",
        "صبور",
        "صبيح",
        "صخر",
        "صدر الدّين",
        "صدقي",
        "صدّاح",
        "صدّام",
        "صعب",
        "صقر",
        "صلاح",
        "صلاح الدّين",
        "صنديد",
        "صهيب",
        "ضاحك",
        "ضاحي",
        "ضحّاك",
        "ضرغام",
        "ضياء",
        "ضياء الدّين",
        "ضيائي",
        "طائع",
        "طائف",
        "طائل",
        "طارق",
        "طالب",
        "طامح",
        "طاهر",
        "طبّاع",
        "طريف",
        "طلال",
        "طلعت",
        "طموح",
        "طه",
        "طيّب",
        "طيّع",
        "ظاعن",
        "ظافر",
        "ظاهر",
        "ظبي",
        "ظريف",
        "ظهير",
        "عائد",
        "عابد",
        "عاتب",
        "عادل",
        "عارف",
        "عاصم",
        "عاطف",
        "عاقل",
        "عاكف",
        "عالم",
        "عامر",
        "عبد الإله",
        "عبد الباري",
        "عبد الباقي",
        "عبد التّواب",
        "عبد الجبّار",
        "عبد الجليل",
        "عبد الحفيظ",
        "عبد الحقّ",
        "عبد الحكيم",
        "عبد الحليم",
        "عبد الحيّ",
        "عبد الخالق",
        "عبد الرّؤوف",
        "عبد الرّحمن",
        "عبد الرّحيم",
        "عبد الرّزاق",
        "عبد الرّشيد",
        "عبد السّلام",
        "عبد السّميع",
        "عبد الشّكور",
        "عبد الصّمد",
        "عبد العزيز",
        "عبد العليم",
        "عبد الغفور",
        "عبد الغفّار",
        "عبد الغني",
        "عبد القادر",
        "عبد القدّوس",
        "عبد القهّار",
        "عبد الكريم",
        "عبد اللطيف",
        "عبد المجيد",
        "عبد المحيي",
        "عبد الملك",
        "عبد المولى",
        "عبد الواحد",
        "عبدالرّحمن",
        "عبدالله",
        "عبّاس",
        "عبّود",
        "عتريس",
        "عتيد",
        "عتيق",
        "عثمان",
        "عدلي",
        "عدنان",
        "عدوي",
        "عذب",
        "عربي",
        "عرفات",
        "عرفان",
        "عرفه",
        "عزاز",
        "عزمي",
        "عزيز",
        "عزّ الدّين",
        "عزّت",
        "عصام",
        "عصمت",
        "عطاء",
        "عفيف",
        "عقيل",
        "علاء",
        "علاء الدّين",
        "علم الدّين",
        "علوان",
        "علي",
        "علّام",
        "عماد",
        "عمر",
        "عمران",
        "عمرو",
        "عمير",
        "عمّار",
        "غازي",
        "غالب",
        "غالي",
        "غامد",
        "غانم",
        "غزوان",
        "غزير",
        "غسّان",
        "غطفان",
        "فؤاد",
        "فائق",
        "فاتح",
        "فاخر",
        "فادي",
        "فارس",
        "فارع",
        "فاروق",
        "فاضل",
        "فالح",
        "فايد",
        "فتحي",
        "فتوح",
        "فخر",
        "فخر الدّين",
        "فخري",
        "فداء",
        "فدائي",
        "فراس",
        "فرج",
        "فرحان",
        "فرزدق",
        "فضل",
        "فطين",
        "فكري",
        "فلاح",
        "فهد",
        "فهمي",
        "فوزي",
        "فوّاز",
        "فيصل",
        "فيّاض",
        "قائد",
        "قاسم",
        "قاصد",
        "قانت",
        "قبس",
        "قحطان",
        "قدري",
        "قصي",
        "قصيد",
        "قطب",
        "قطز",
        "قنوع",
        "قيس",
        "كارم",
        "كاسر",
        "كاشف",
        "كاظم",
        "كافور",
        "كامل",
        "كايد",
        "كبير",
        "كتوم",
        "كرم",
        "كريم",
        "كسّاب",
        "كليم",
        "كمال",
        "كنار",
        "كنان",
        "كنعان",
        "لؤي",
        "لبيب",
        "لبيد",
        "لطفي",
        "لطوف",
        "لفيف",
        "لقاء",
        "لقمان",
        "لمّاح",
        "لهفان",
        "ليث",
        "مأمون",
        "مؤمن",
        "مؤنس",
        "مؤيّد",
        "ماجد",
        "مازن",
        "مالك",
        "ماهر",
        "مجاهد",
        "مجد",
        "محجوب",
        "محسن",
        "محفوظ",
        "محمود",
        "محمّد",
        "محيي الدّين",
        "مختار",
        "مخلص",
        "مدحت",
        "مراد",
        "مرادي",
        "مرتجي",
        "مرتضي",
        "مرتقي",
        "مرزوق",
        "مرسال",
        "مرشد",
        "مرعي",
        "مروان",
        "مزهر",
        "مسرور",
        "مسعود",
        "مسلم",
        "مشاري",
        "مشرف",
        "مشرق",
        "مشفق",
        "مصباح",
        "مصطفى",
        "مصعب",
        "مطاوع",
        "مظهر",
        "معارف",
        "معتوق",
        "معزّ",
        "معمّر",
        "معن",
        "معين",
        "مفيد",
        "مقداد",
        "مقدام",
        "مكرّم",
        "مكّي",
        "ملهم",
        "ممتاز",
        "ممدوح",
        "مناف",
        "منتصر",
        "منسي",
        "منصور",
        "منيب",
        "منيع",
        "منيف",
        "مهدي",
        "مهران",
        "مهنّد",
        "مهيب",
        "موسى",
        "موفّق",
        "مياس",
        "ميثاق",
        "ميسور",
        "ميمون",
        "ميّاد",
        "مَجدي",
        "مَسعد",
        "مُتعب",
        "مُتوكّل",
        "مُتولي",
        "مُتيّم",
        "مُخيمر",
        "مُرسي",
        "مُرضي",
        "مُسعف",
        "مُصلح",
        "مُعتز",
        "مُناضل",
        "مُنجد",
        "مُنذر",
        "مُنير",
        "نائل",
        "ناجح",
        "ناجي",
        "نادر",
        "نادي",
        "ناصر",
        "ناصر الدّين",
        "ناصيف",
        "ناضر",
        "ناظم",
        "ناعم",
        "نافذ",
        "نافع",
        "نبراس",
        "نبهان",
        "نبيل",
        "نبيه",
        "نجدت",
        "نجم الدّين",
        "نجوان",
        "نجيب",
        "نديم",
        "نذير",
        "نزار",
        "نزيه",
        "نسيب",
        "نشأت",
        "نشوان",
        "نصر",
        "نصر الدّين",
        "نصري",
        "نصوح",
        "نصور",
        "نضال",
        "نظام",
        "نظمي",
        "نعمان",
        "نعيم",
        "نمر",
        "نوح",
        "نور",
        "نور الدّين",
        "نور الحقّ",
        "نورس",
        "نوري",
        "نوّار",
        "نوّاف",
        "نيازي",
        "هادي",
        "هاشم",
        "هاني",
        "هايل",
        "هزار",
        "هلال",
        "هلالي",
        "همام",
        "هيثم",
        "هيكل",
        "هيمان",
        "وائل",
        "واثق",
        "وادع",
        "واصف",
        "واصل",
        "وثّاب",
        "وجدي",
        "وجيه",
        "وحيد",
        "ودود",
        "وديع",
        "وريد",
        "وسام",
        "وسيل",
        "وسيم",
        "وصفي",
        "وضّاح",
        "وفائي",
        "وفيق",
        "وليد",
        "وليف",
        "ياسر",
        "يافع",
        "ياقوت",
        "يانع",
        "يحيى",
        "يزيد",
        "يسار",
        "يسري",
        "يعرب",
        "يعقوب",
        "يقين",
        "يمام",
        "يوسف",
        "يونس",
    )

    first_names = first_names_male + first_names_female

    last_names: Tuple[str, ...] = (
        "أشجع",
        "أفغاني",
        "أكلب",
        "ألمع",
        "أنمار",
        "أولاد بوعزيز",
        "أولاد زيان",
        "إياد",
        "ابو الحاج",
        "ابو السعود",
        "ابو عيد",
        "ارناؤوط",
        "ازحيمان",
        "اسطمبولي",
        "الأزد",
        "الأشراف",
        "الألجاوي",
        "الأنصاري",
        "الأوس",
        "الأيوبي",
        "الامام",
        "البامية",
        "البخاري",
        "البديري",
        "البشيتي",
        "البغدادي",
        "البقوم",
        "البيسار القعقور",
        "البيطار",
        "الترجمان الصالح",
        "الترهي",
        "التوتنجي",
        "الجاعوني",
        "الجبشة",
        "الجعليين",
        "الحجر بن الهنوء بن الأزد",
        "الحداء",
        "الحسيني",
        "الحكم بن سعد العشيرة",
        "الحلاق",
        "الحلواني",
        "الحواش",
        "الحويطات",
        "الخالدي",
        "الخزرج",
        "الخطيب بني جماعة الكناني",
        "الخلفاوي",
        "الداودي",
        "الدجاني",
        "الدسوقي",
        "الدقاق",
        "الدليم",
        "الدواسر",
        "الديسي",
        "الرباب",
        "الرباطاب",
        "الزرقان",
        "الزماميري",
        "الساحلي",
        "السادة",
        "السادة الراويون",
        "السروري",
        "السمان",
        "السهول",
        "السيفي",
        "الشامي",
        "الشاويش",
        "الشايقية",
        "الشحوح",
        "الشرفاء",
        "الشعباني",
        "الشهابي",
        "الطحان",
        "الظفير",
        "العارف",
        "العجمان",
        "العسلي",
        "العفيفي",
        "العقيدات",
        "العلمي",
        "العوازم",
        "العوالق",
        "الغوانمة",
        "الفتياني",
        "القاعي",
        "القباني",
        "القرجولي",
        "القزاز",
        "القضماني",
        "القطب",
        "القلموني",
        "القواسم",
        "الكالوتي",
        "الكبابيش",
        "الكثيري",
        "الكلغاصي",
        "الكواهلة",
        "المؤقت",
        "الماني",
        "المتولي",
        "المرازيق",
        "المظفر",
        "المغربي",
        "المفتي",
        "المملوك",
        "المنتفق",
        "المهرة",
        "الموركة",
        "الموسوس",
        "النجار",
        "النشاشيبي",
        "النقيب",
        "النمر",
        "النمري",
        "الهدمي",
        "الوعري",
        "اليوزباشي",
        "اميوني",
        "اهرام",
        "بارق",
        "باهلة",
        "بتروني",
        "بجيلة",
        "بحمدوني",
        "بدرية",
        "بديرية",
        "بعلبكي",
        "بكر بن عبد مناة",
        "بكر بن وائل",
        "بكيل",
        "بلغازي",
        "بلقرن",
        "بلي",
        "بنو أسد",
        "بنو أمية",
        "بنو الأحمر",
        "بنو الأحمر بن الحارث",
        "بنو الأسمر",
        "بنو الحارث بن كعب",
        "بنو الدئل",
        "بنو العريج",
        "بنو النجار",
        "بنو حنيفة",
        "بنو خالد",
        "بنو ذي أصبح",
        "بنو زيد",
        "بنو سعد بن بكر",
        "بنو سعد بن ليث بن بكر",
        "بنو شعبة",
        "بنو شهر",
        "بنو شيبان",
        "بنو شيبة",
        "بنو صخر",
        "بنو ضمرة",
        "بنو عبس",
        "بنو عجل",
        "بنو عدي",
        "بنو عمرو",
        "بنو فراس",
        "بنو كلب",
        "بنو كنز",
        "بنو لام",
        "بنو ليث",
        "بنو مالك",
        "بنو معقل",
        "بنو مهدي",
        "بنو هاشم",
        "بنو هلال",
        "بنو ياس",
        "بنو يعلى",
        "بني بيات",
        "بني رشيد",
        "بني عطية",
        "بني هاجر",
        "بو مدين",
        "بيرقدار",
        "بيروتي",
        "ترابين",
        "تغلب بن وائل",
        "تميم",
        "تنوخ",
        "ثقيف",
        "جار الله",
        "جبيلي",
        "جديس",
        "جذام",
        "جرهم",
        "جزار",
        "جزيني",
        "جعفر",
        "جهينة",
        "جودة",
        "حاشد",
        "حب رمان",
        "حجازي",
        "حرب",
        "حمير",
        "حميضة",
        "حوالة",
        "خثعم",
        "خزاعة",
        "خندف",
        "خولان",
        "درويش",
        "دوبلال",
        "راجح",
        "ربيعة",
        "رصاص",
        "زبيد",
        "زحلاوي",
        "زحيكة",
        "زلاطيمو",
        "زهران",
        "سبيع",
        "سرندح",
        "سليم",
        "سموم",
        "سوميرة",
        "شتية",
        "شرف",
        "شمر",
        "شمران",
        "شهران",
        "شويفاتي",
        "صيام",
        "صيداني",
        "صيداوي",
        "ضبيعة",
        "طرابلسي",
        "طزيز",
        "طسم",
        "طقش",
        "طه",
        "طوطح",
        "طيء",
        "عامر بن صعصعة",
        "عاملة",
        "عبد القيس",
        "عبد اللطيف",
        "عبده",
        "عتيبة",
        "عجرمة (العجارمة)",
        "عدوان",
        "عذرة",
        "عرموني",
        "عسير",
        "عضل",
        "عكاوي",
        "عليان",
        "عنز بن وائل",
        "عنزة",
        "عنس",
        "عويضة",
        "غامد",
        "غطفان",
        "غنيم",
        "غوشة",
        "فراهيد",
        "فهم",
        "قبيلة هذيل البقوم",
        "قحطان",
        "قرش",
        "قريش",
        "قضاعة",
        "قطينة",
        "قليبو",
        "قيس عيلان",
        "كمال",
        "كنانة",
        "كندة",
        "كهلان",
        "لخم",
        "متني",
        "مذحج",
        "مراد",
        "مرازيق البقوم",
        "مزرعاني",
        "مزينة",
        "مشعشع",
        "مضر",
        "مطير",
        "معتوق",
        "ميرفاب",
        "نجم",
        "نجيب",
        "نسيبة",
        "نهد",
        "نور الدين",
        "هذيل",
        "همدان",
        "هندية",
        "هوازن",
        "وهبة",
        "يافع",
        "يشكر",
    )

    prefixes_female: Tuple[str, ...] = (
        "السيدة",
        "الآنسة",
        "الدكتورة",
        "الأستاذة",
        "المهندسة",
    )
    prefixes_male: Tuple[str, ...] = ("السيد", "المهندس", "الدكتور", "الأستاذ")



"""Internal module for human-friendly color generation.

.. important::
   End users of this library should not use anything in this module.

Code adapted from:
- https://github.com/davidmerfield/randomColor  (CC0)
- https://github.com/kevinwuhoo/randomcolor-py  (MIT License)

Additional reference from:
- https://en.wikipedia.org/wiki/HSL_and_HSV
"""

import colorsys
import math
import random
import sys

from typing import TYPE_CHECKING, Dict, Hashable, Optional, Sequence, Tuple

if TYPE_CHECKING:
    from ...factory import Generator

from ...typing import HueType

COLOR_MAP: Dict[str, Dict[str, Sequence[Tuple[int, int]]]] = {
    "monochrome": {
        "hue_range": [(0, 0)],
        "lower_bounds": [
            (0, 0),
            (100, 0),
        ],
    },
    "red": {
        "hue_range": [(-26, 18)],
        "lower_bounds": [
            (20, 100),
            (30, 92),
            (40, 89),
            (50, 85),
            (60, 78),
            (70, 70),
            (80, 60),
            (90, 55),
            (100, 50),
        ],
    },
    "orange": {
        "hue_range": [(19, 46)],
        "lower_bounds": [
            (20, 100),
            (30, 93),
            (40, 88),
            (50, 86),
            (60, 85),
            (70, 70),
            (100, 70),
        ],
    },
    "yellow": {
        "hue_range": [(47, 62)],
        "lower_bounds": [
            (25, 100),
            (40, 94),
            (50, 89),
            (60, 86),
            (70, 84),
            (80, 82),
            (90, 80),
            (100, 75),
        ],
    },
    "green": {
        "hue_range": [(63, 178)],
        "lower_bounds": [
            (30, 100),
            (40, 90),
            (50, 85),
            (60, 81),
            (70, 74),
            (80, 64),
            (90, 50),
            (100, 40),
        ],
    },
    "blue": {
        "hue_range": [(179, 257)],
        "lower_bounds": [
            (20, 100),
            (30, 86),
            (40, 80),
            (50, 74),
            (60, 60),
            (70, 52),
            (80, 44),
            (90, 39),
            (100, 35),
        ],
    },
    "purple": {
        "hue_range": [(258, 282)],
        "lower_bounds": [
            (20, 100),
            (30, 87),
            (40, 79),
            (50, 70),
            (60, 65),
            (70, 59),
            (80, 52),
            (90, 45),
            (100, 42),
        ],
    },
    "pink": {
        "hue_range": [(283, 334)],
        "lower_bounds": [
            (20, 100),
            (30, 90),
            (40, 86),
            (60, 84),
            (80, 80),
            (90, 75),
            (100, 73),
        ],
    },
}


class RandomColor:
    """Implement random color generation in a human-friendly way.

    This helper class encapsulates the internal implementation and logic of the
    :meth:`color() <faker.providers.color.Provider.color>` method.
    """

    def __init__(self, generator: Optional["Generator"] = None, seed: Optional[Hashable] = None) -> None:
        self.colormap = COLOR_MAP

        # Option to specify a seed was not removed so this class
        # can still be tested independently w/o generators
        if generator:
            self.random = generator.random
        else:
            self.seed = seed if seed else random.randint(0, sys.maxsize)
            self.random = random.Random(self.seed)

        for color_name, color_attrs in self.colormap.items():
            lower_bounds: Sequence[Tuple[int, int]] = color_attrs["lower_bounds"]
            s_min, b_max = lower_bounds[0]
            s_max, b_min = lower_bounds[-1]

            self.colormap[color_name]["saturation_range"] = [(s_min, s_max)]
            self.colormap[color_name]["brightness_range"] = [(b_min, b_max)]

    def generate(
        self,
        hue: Optional[HueType] = None,
        luminosity: Optional[str] = None,
        color_format: str = "hex",
    ) -> str:
        """Generate a color.

        Whenever :meth:`color() <faker.providers.color.Provider.color>` is
        called, the arguments used are simply passed into this method, and this
        method handles the rest.
        """
        # First we pick a hue (H)
        h = self.pick_hue(hue)

        # Then use H to determine saturation (S)
        s = self.pick_saturation(h, hue, luminosity)

        # Then use S and H to determine brightness (B).
        b = self.pick_brightness(h, s, luminosity)

        # Then we return the HSB color in the desired format
        return self.set_format((h, s, b), color_format)

    def pick_hue(self, hue: Optional[HueType]) -> int:
        """Return a numerical hue value."""
        hue_ = self.random_within(self.get_hue_range(hue))

        # Instead of storing red as two separate ranges,
        # we group them, using negative numbers
        if hue_ < 0:
            hue_ += 360

        return hue_

    def pick_saturation(self, hue: int, hue_name: Optional[HueType], luminosity: Optional[str]) -> int:
        """Return a numerical saturation value."""
        if luminosity is None:
            luminosity = ""
        if luminosity == "random":
            return self.random_within((0, 100))

        if isinstance(hue_name, str) and hue_name == "monochrome":
            return 0

        s_min, s_max = self.get_saturation_range(hue)

        if luminosity == "bright":
            s_min = 55
        elif luminosity == "dark":
            s_min = s_max - 10
        elif luminosity == "light":
            s_max = 55

        return self.random_within((s_min, s_max))

    def pick_brightness(self, h: int, s: int, luminosity: Optional[str]) -> int:
        """Return a numerical brightness value."""
        if luminosity is None:
            luminosity = ""

        b_min = self.get_minimum_brightness(h, s)
        b_max = 100

        if luminosity == "dark":
            b_max = b_min + 20
        elif luminosity == "light":
            b_min = (b_max + b_min) // 2
        elif luminosity == "random":
            b_min = 0
            b_max = 100

        return self.random_within((b_min, b_max))

    def set_format(self, hsv: Tuple[int, int, int], color_format: str) -> str:
        """Handle conversion of HSV values into desired format."""
        if color_format == "hsv":
            color = f"hsv({hsv[0]}, {hsv[1]}, {hsv[2]})"

        elif color_format == "hsl":
            hsl = self.hsv_to_hsl(hsv)
            color = f"hsl({hsl[0]}, {hsl[1]}, {hsl[2]})"

        elif color_format == "rgb":
            rgb = self.hsv_to_rgb(hsv)
            color = f"rgb({rgb[0]}, {rgb[1]}, {rgb[2]})"

        else:
            rgb = self.hsv_to_rgb(hsv)
            color = f"#{rgb[0]:02x}{rgb[1]:02x}{rgb[2]:02x}"

        return color

    def get_minimum_brightness(self, h: int, s: int) -> int:
        """Return the minimum allowed brightness for ``h`` and ``s``."""
        lower_bounds: Sequence[Tuple[int, int]] = self.get_color_info(h)["lower_bounds"]

        for i in range(len(lower_bounds) - 1):
            s1, v1 = lower_bounds[i]
            s2, v2 = lower_bounds[i + 1]

            if s1 <= s <= s2:
                m: float = (v2 - v1) / (s2 - s1)
                b: float = v1 - m * s1

                return int(m * s + b)

        return 0

    def get_hue_range(self, color_input: Optional[HueType]) -> Tuple[int, int]:
        """Return the hue range for a given ``color_input``."""
        if isinstance(color_input, (int, float)) and 0 <= color_input <= 360:
            color_input = int(color_input)
            return (color_input, color_input)
        elif isinstance(color_input, str) and color_input in self.colormap:
            return self.colormap[color_input]["hue_range"][0]
        elif color_input is None:
            return (0, 360)

        if isinstance(color_input, list):
            color_input = tuple(color_input)
        if (
            isinstance(color_input, tuple)
            and len(color_input) == 2
            and all(isinstance(c, (float, int)) for c in color_input)
        ):
            v1 = int(color_input[0])
            v2 = int(color_input[1])

            if v2 < v1:
                v1, v2 = v2, v1
            v1 = max(v1, 0)
            v2 = min(v2, 360)
            return (v1, v2)
        raise TypeError("Hue must be a valid string, numeric type, or a tuple/list of 2 numeric types.")

    def get_saturation_range(self, hue: int) -> Tuple[int, int]:
        """Return the saturation range for a given numerical ``hue`` value."""
        return self.get_color_info(hue)["saturation_range"][0]

    def get_color_info(self, hue: int) -> Dict[str, Sequence[Tuple[int, int]]]:
        """Return the color info for a given numerical ``hue`` value."""
        # Maps red colors to make picking hue easier
        if 334 <= hue <= 360:
            hue -= 360

        for color_name, color in self.colormap.items():
            hue_range: Tuple[int, int] = color["hue_range"][0]
            if hue_range[0] <= hue <= hue_range[1]:
                return self.colormap[color_name]
        else:
            raise ValueError("Value of hue `%s` is invalid." % hue)

    def random_within(self, r: Sequence[int]) -> int:
        """Return a random integer within the range ``r``."""
        return self.random.randint(int(r[0]), int(r[1]))

    @classmethod
    def hsv_to_rgb(cls, hsv: Tuple[int, int, int]) -> Tuple[int, int, int]:
        """Convert HSV to RGB.

        This method expects ``hsv`` to be a 3-tuple of H, S, and V values, and
        it will return a 3-tuple of the equivalent R, G, and B values.
        """
        h, s, v = hsv
        h = max(h, 1)
        h = min(h, 359)

        r, g, b = colorsys.hsv_to_rgb(h / 360, s / 100, v / 100)
        return (int(r * 255), int(g * 255), int(b * 255))

    @classmethod
    def hsv_to_hsl(cls, hsv: Tuple[int, int, int]) -> Tuple[int, int, int]:
        """Convert HSV to HSL.

        This method expects ``hsv`` to be a 3-tuple of H, S, and V values, and
        it will return a 3-tuple of the equivalent H, S, and L values.
        """
        h, s, v = hsv

        s_: float = s / 100.0
        v_: float = v / 100.0
        l = 0.5 * (v_) * (2 - s_)  # noqa: E741

        s_ = 0.0 if l in [0, 1] else v_ * s_ / (1 - math.fabs(2 * l - 1))
        return (int(h), int(s_ * 100), int(l * 100))



import warnings

from typing import Any

from ..fr_CA import Provider as FRCAProvider


class Provider(FRCAProvider):
    def __init__(self, *args: Any, **kwargs: Any) -> None:
        warnings.warn("fr_QC locale is deprecated. Please use fr_CA.")
        super().__init__(*args, **kwargs)



import os.path


class AllureData:

    def __init__(self, report_path):
        self.report_path = report_path

    def get_basic_info(self):
        # 从文件中读取数据
        prometheus_data_path = os.path.join(self.report_path, 'export', 'prometheusData.txt')
        with open(prometheus_data_path, 'r') as f:
            lines = f.readlines()

        # 数据清洗
        summary = {}
        for line in lines:
            line = line.strip('\n')
            data = line.split(' ')
            key = data[0]
            value = data[1]
            summary.update({key: value})

        # 数据结构化
        total = int(summary['launch_retries_run'])
        passed = int(summary['launch_status_passed'])
        fail = total - passed
        rate = int((passed / total) * 100)
        result = {"total": total, "passed": passed, "rate": rate, "fail": fail}

        return result







from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``en_US`` locale.

    Word list is based on the source(s) below, and some words have been removed
    to make the word list appropriate for public testing.

    Sources:

    - http://www.ef.edu/english-resources/english-vocabulary/top-1000-words/
    """

    word_list = (
        "a",
        "ability",
        "able",
        "about",
        "above",
        "accept",
        "according",
        "account",
        "across",
        "act",
        "action",
        "activity",
        "actually",
        "add",
        "address",
        "administration",
        "admit",
        "adult",
        "affect",
        "after",
        "again",
        "against",
        "age",
        "agency",
        "agent",
        "ago",
        "agree",
        "agreement",
        "ahead",
        "air",
        "all",
        "allow",
        "almost",
        "alone",
        "along",
        "already",
        "also",
        "although",
        "always",
        "American",
        "among",
        "amount",
        "analysis",
        "and",
        "animal",
        "another",
        "answer",
        "any",
        "anyone",
        "anything",
        "appear",
        "apply",
        "approach",
        "area",
        "argue",
        "arm",
        "around",
        "arrive",
        "art",
        "article",
        "artist",
        "as",
        "ask",
        "assume",
        "at",
        "attack",
        "attention",
        "attorney",
        "audience",
        "author",
        "authority",
        "available",
        "avoid",
        "away",
        "baby",
        "back",
        "bad",
        "bag",
        "ball",
        "bank",
        "bar",
        "base",
        "be",
        "beat",
        "beautiful",
        "because",
        "become",
        "bed",
        "before",
        "begin",
        "behavior",
        "behind",
        "believe",
        "benefit",
        "best",
        "better",
        "between",
        "beyond",
        "big",
        "bill",
        "billion",
        "bit",
        "black",
        "blood",
        "blue",
        "board",
        "body",
        "book",
        "born",
        "both",
        "box",
        "boy",
        "break",
        "bring",
        "brother",
        "budget",
        "build",
        "building",
        "business",
        "but",
        "buy",
        "by",
        "call",
        "camera",
        "campaign",
        "can",
        "candidate",
        "capital",
        "car",
        "card",
        "care",
        "career",
        "carry",
        "case",
        "catch",
        "cause",
        "cell",
        "center",
        "central",
        "century",
        "certain",
        "certainly",
        "chair",
        "challenge",
        "chance",
        "change",
        "character",
        "charge",
        "check",
        "child",
        "choice",
        "choose",
        "church",
        "citizen",
        "city",
        "civil",
        "claim",
        "class",
        "clear",
        "clearly",
        "close",
        "coach",
        "cold",
        "collection",
        "college",
        "color",
        "commercial",
        "common",
        "community",
        "company",
        "compare",
        "computer",
        "concern",
        "condition",
        "conference",
        "Congress",
        "consider",
        "consumer",
        "contain",
        "continue",
        "control",
        "cost",
        "could",
        "country",
        "couple",
        "course",
        "court",
        "cover",
        "create",
        "crime",
        "cultural",
        "culture",
        "cup",
        "current",
        "customer",
        "cut",
        "dark",
        "data",
        "daughter",
        "day",
        "deal",
        "debate",
        "decade",
        "decide",
        "decision",
        "deep",
        "defense",
        "degree",
        "Democrat",
        "democratic",
        "describe",
        "design",
        "despite",
        "detail",
        "determine",
        "develop",
        "development",
        "difference",
        "different",
        "difficult",
        "dinner",
        "direction",
        "director",
        "discover",
        "discuss",
        "discussion",
        "do",
        "doctor",
        "dog",
        "door",
        "down",
        "draw",
        "dream",
        "drive",
        "drop",
        "drug",
        "during",
        "each",
        "early",
        "east",
        "easy",
        "eat",
        "economic",
        "economy",
        "edge",
        "education",
        "effect",
        "effort",
        "eight",
        "either",
        "election",
        "else",
        "employee",
        "end",
        "energy",
        "enjoy",
        "enough",
        "enter",
        "entire",
        "environment",
        "environmental",
        "especially",
        "establish",
        "even",
        "evening",
        "event",
        "ever",
        "every",
        "everybody",
        "everyone",
        "everything",
        "evidence",
        "exactly",
        "example",
        "executive",
        "exist",
        "expect",
        "experience",
        "expert",
        "explain",
        "eye",
        "face",
        "fact",
        "factor",
        "fall",
        "family",
        "far",
        "fast",
        "father",
        "fear",
        "federal",
        "feel",
        "feeling",
        "few",
        "field",
        "fight",
        "figure",
        "fill",
        "film",
        "final",
        "finally",
        "financial",
        "find",
        "fine",
        "finish",
        "fire",
        "firm",
        "first",
        "fish",
        "five",
        "floor",
        "fly",
        "focus",
        "follow",
        "food",
        "foot",
        "for",
        "force",
        "foreign",
        "forget",
        "form",
        "former",
        "forward",
        "four",
        "free",
        "friend",
        "from",
        "front",
        "full",
        "fund",
        "future",
        "game",
        "garden",
        "gas",
        "general",
        "generation",
        "get",
        "girl",
        "give",
        "glass",
        "go",
        "goal",
        "good",
        "government",
        "great",
        "green",
        "ground",
        "group",
        "grow",
        "growth",
        "guess",
        "gun",
        "guy",
        "hair",
        "half",
        "hand",
        "happen",
        "happy",
        "hard",
        "have",
        "he",
        "head",
        "health",
        "hear",
        "heart",
        "heavy",
        "help",
        "her",
        "here",
        "herself",
        "high",
        "him",
        "himself",
        "his",
        "history",
        "hit",
        "hold",
        "home",
        "hope",
        "hospital",
        "hot",
        "hotel",
        "hour",
        "house",
        "how",
        "however",
        "huge",
        "human",
        "hundred",
        "husband",
        "I",
        "idea",
        "identify",
        "if",
        "image",
        "imagine",
        "impact",
        "important",
        "improve",
        "in",
        "include",
        "including",
        "increase",
        "indeed",
        "indicate",
        "individual",
        "industry",
        "information",
        "inside",
        "instead",
        "institution",
        "interest",
        "interesting",
        "international",
        "interview",
        "into",
        "investment",
        "involve",
        "issue",
        "it",
        "item",
        "its",
        "itself",
        "job",
        "join",
        "just",
        "keep",
        "key",
        "kid",
        "kind",
        "kitchen",
        "know",
        "knowledge",
        "land",
        "language",
        "large",
        "last",
        "late",
        "later",
        "laugh",
        "law",
        "lawyer",
        "lay",
        "lead",
        "leader",
        "learn",
        "least",
        "leave",
        "left",
        "leg",
        "less",
        "let",
        "letter",
        "level",
        "life",
        "light",
        "like",
        "likely",
        "line",
        "list",
        "listen",
        "little",
        "live",
        "local",
        "long",
        "look",
        "lose",
        "loss",
        "lot",
        "low",
        "machine",
        "magazine",
        "main",
        "maintain",
        "major",
        "majority",
        "make",
        "man",
        "manage",
        "management",
        "manager",
        "many",
        "market",
        "marriage",
        "material",
        "matter",
        "may",
        "maybe",
        "me",
        "mean",
        "measure",
        "media",
        "medical",
        "meet",
        "meeting",
        "member",
        "memory",
        "mention",
        "message",
        "method",
        "middle",
        "might",
        "military",
        "million",
        "mind",
        "minute",
        "miss",
        "mission",
        "model",
        "modern",
        "moment",
        "money",
        "month",
        "more",
        "morning",
        "most",
        "mother",
        "mouth",
        "move",
        "movement",
        "movie",
        "Mr",
        "Mrs",
        "much",
        "music",
        "must",
        "my",
        "myself",
        "name",
        "nation",
        "national",
        "natural",
        "nature",
        "near",
        "nearly",
        "necessary",
        "need",
        "network",
        "never",
        "new",
        "news",
        "newspaper",
        "next",
        "nice",
        "night",
        "no",
        "none",
        "nor",
        "north",
        "not",
        "note",
        "nothing",
        "notice",
        "now",
        "number",
        "occur",
        "of",
        "off",
        "offer",
        "office",
        "officer",
        "official",
        "often",
        "oil",
        "ok",
        "old",
        "on",
        "once",
        "one",
        "only",
        "onto",
        "open",
        "operation",
        "opportunity",
        "option",
        "or",
        "order",
        "organization",
        "other",
        "others",
        "our",
        "out",
        "outside",
        "over",
        "own",
        "owner",
        "page",
        "painting",
        "paper",
        "parent",
        "part",
        "participant",
        "particular",
        "particularly",
        "partner",
        "party",
        "pass",
        "past",
        "pattern",
        "pay",
        "peace",
        "people",
        "per",
        "perform",
        "performance",
        "perhaps",
        "person",
        "personal",
        "phone",
        "physical",
        "pick",
        "picture",
        "piece",
        "place",
        "plan",
        "plant",
        "play",
        "player",
        "PM",
        "point",
        "police",
        "policy",
        "political",
        "politics",
        "poor",
        "popular",
        "population",
        "position",
        "positive",
        "possible",
        "power",
        "practice",
        "prepare",
        "present",
        "president",
        "pressure",
        "pretty",
        "prevent",
        "price",
        "probably",
        "process",
        "produce",
        "product",
        "production",
        "professional",
        "professor",
        "program",
        "project",
        "property",
        "protect",
        "prove",
        "provide",
        "public",
        "pull",
        "purpose",
        "push",
        "put",
        "quality",
        "question",
        "quickly",
        "quite",
        "race",
        "radio",
        "raise",
        "range",
        "rate",
        "rather",
        "reach",
        "read",
        "ready",
        "real",
        "reality",
        "realize",
        "really",
        "reason",
        "receive",
        "recent",
        "recently",
        "recognize",
        "record",
        "red",
        "reduce",
        "reflect",
        "region",
        "relate",
        "relationship",
        "religious",
        "remain",
        "remember",
        "report",
        "represent",
        "Republican",
        "require",
        "research",
        "resource",
        "respond",
        "response",
        "responsibility",
        "rest",
        "result",
        "return",
        "reveal",
        "rich",
        "right",
        "rise",
        "risk",
        "road",
        "rock",
        "role",
        "room",
        "rule",
        "run",
        "safe",
        "same",
        "save",
        "say",
        "scene",
        "school",
        "science",
        "scientist",
        "score",
        "sea",
        "season",
        "seat",
        "second",
        "section",
        "security",
        "see",
        "seek",
        "seem",
        "sell",
        "send",
        "senior",
        "sense",
        "series",
        "serious",
        "serve",
        "service",
        "set",
        "seven",
        "several",
        "shake",
        "share",
        "she",
        "short",
        "should",
        "shoulder",
        "show",
        "side",
        "sign",
        "significant",
        "similar",
        "simple",
        "simply",
        "since",
        "sing",
        "single",
        "sister",
        "sit",
        "site",
        "situation",
        "six",
        "size",
        "skill",
        "skin",
        "small",
        "smile",
        "so",
        "social",
        "society",
        "soldier",
        "some",
        "somebody",
        "someone",
        "something",
        "sometimes",
        "son",
        "song",
        "soon",
        "sort",
        "sound",
        "source",
        "south",
        "southern",
        "space",
        "speak",
        "special",
        "specific",
        "speech",
        "spend",
        "sport",
        "spring",
        "staff",
        "stage",
        "stand",
        "standard",
        "star",
        "start",
        "state",
        "statement",
        "station",
        "stay",
        "step",
        "still",
        "stock",
        "stop",
        "store",
        "story",
        "strategy",
        "street",
        "strong",
        "structure",
        "student",
        "study",
        "stuff",
        "style",
        "subject",
        "success",
        "successful",
        "such",
        "suddenly",
        "suffer",
        "suggest",
        "summer",
        "support",
        "sure",
        "surface",
        "system",
        "table",
        "take",
        "talk",
        "task",
        "tax",
        "teach",
        "teacher",
        "team",
        "technology",
        "television",
        "tell",
        "ten",
        "tend",
        "term",
        "test",
        "than",
        "thank",
        "that",
        "the",
        "their",
        "them",
        "themselves",
        "then",
        "theory",
        "there",
        "these",
        "they",
        "thing",
        "think",
        "third",
        "this",
        "those",
        "though",
        "thought",
        "thousand",
        "threat",
        "three",
        "through",
        "throughout",
        "throw",
        "thus",
        "time",
        "to",
        "today",
        "together",
        "tonight",
        "too",
        "top",
        "total",
        "tough",
        "toward",
        "town",
        "trade",
        "traditional",
        "training",
        "travel",
        "treat",
        "treatment",
        "tree",
        "trial",
        "trip",
        "trouble",
        "true",
        "truth",
        "try",
        "turn",
        "TV",
        "two",
        "type",
        "under",
        "understand",
        "unit",
        "until",
        "up",
        "upon",
        "us",
        "use",
        "usually",
        "value",
        "various",
        "very",
        "view",
        "visit",
        "voice",
        "vote",
        "wait",
        "walk",
        "wall",
        "want",
        "war",
        "watch",
        "water",
        "way",
        "we",
        "wear",
        "week",
        "weight",
        "well",
        "west",
        "western",
        "what",
        "whatever",
        "when",
        "where",
        "whether",
        "which",
        "while",
        "white",
        "who",
        "whole",
        "whom",
        "whose",
        "why",
        "wide",
        "wife",
        "will",
        "win",
        "wind",
        "window",
        "wish",
        "with",
        "within",
        "without",
        "woman",
        "wonder",
        "word",
        "work",
        "worker",
        "world",
        "worry",
        "would",
        "write",
        "writer",
        "wrong",
        "yard",
        "yeah",
        "year",
        "yes",
        "yet",
        "you",
        "young",
        "your",
        "yourself",
    )



import datetime
import operator

from typing import List, Optional, Sequence

from ....typing import GenderType
from .. import Provider as SsnProvider


def checksum(digits: Sequence[int], scale: List[int]) -> int:
    """
    Calculate checksum of Norwegian personal identity code.

    Checksum is calculated with "Module 11" method using a scale.
    The digits of the personal code are multiplied by the corresponding
    number in the scale and summed;
    if remainder of module 11 of the sum is less than 10, checksum is the
    remainder.
    If remainder is 0, the checksum is 0.

    https://no.wikipedia.org/wiki/F%C3%B8dselsnummer
    """
    chk_nbr = 11 - (sum(map(operator.mul, digits, scale)) % 11)
    if chk_nbr == 11:
        return 0
    return chk_nbr


class Provider(SsnProvider):
    scale1 = (3, 7, 6, 1, 8, 9, 4, 5, 2)
    scale2 = (5, 4, 3, 2, 7, 6, 5, 4, 3, 2)

    def ssn(self, dob: Optional[str] = None, gender: Optional[GenderType] = None) -> str:
        """
        Returns 11 character Norwegian personal identity code (Fødselsnummer).

        A Norwegian personal identity code consists of 11 digits, without any
        whitespace or other delimiters. The form is DDMMYYIIICC, where III is
        a serial number separating persons born oh the same date with different
        intervals depending on the year they are born. CC is two checksums.
        https://en.wikipedia.org/wiki/National_identification_number#Norway

        :param dob: date of birth as a "YYYYMMDD" string
        :type dob: str
        :param gender: gender of the person - "F" for female, M for male.
        :type gender: str
        :return: Fødselsnummer in str format (11 digs)
        :rtype: str
        """

        if dob:
            birthday = datetime.datetime.strptime(dob, "%Y%m%d")
        else:
            age = datetime.timedelta(days=self.generator.random.randrange(18 * 365, 90 * 365))
            birthday = datetime.datetime.now() - age
        if not gender:
            gender = self.generator.random.choice(("F", "M"))
        elif gender not in ("F", "M"):
            raise ValueError("Gender must be one of F or M.")

        while True:
            if 1900 <= birthday.year <= 1999:
                suffix = str(self.generator.random.randrange(0, 49))
            elif 1854 <= birthday.year <= 1899:
                suffix = str(self.generator.random.randrange(50, 74))
            elif 2000 <= birthday.year <= 2039:
                suffix = str(self.generator.random.randrange(50, 99))
            elif 1940 <= birthday.year <= 1999:
                suffix = str(self.generator.random.randrange(90, 99))
            if gender == "F":
                gender_num = self.generator.random.choice((0, 2, 4, 6, 8))
            elif gender == "M":
                gender_num = self.generator.random.choice((1, 3, 5, 7, 9))
            pnr = birthday.strftime("%d%m%y") + suffix.zfill(2) + str(gender_num)
            pnr_nums = [int(ch) for ch in pnr]
            k1 = checksum(Provider.scale1, pnr_nums)
            k2 = checksum(Provider.scale2, pnr_nums + [k1])
            # Checksums with a value of 10 is rejected.
            # https://no.wikipedia.org/wiki/F%C3%B8dselsnummer
            if k1 == 10 or k2 == 10:
                continue
            pnr += f"{k1}{k2}"
            return pnr



import argparse
import sys
from qrunner import __version__, __description__
from qrunner.scaffold import init_scaffold_project, main_scaffold_project


def main():
    """ API test: parse command line options and run commands.
    """
    parser = argparse.ArgumentParser(description=__description__)
    parser.add_argument(
        "-V", "--version", dest="version", action="store_true", help="show version",
    )

    subparsers = parser.add_subparsers(help="sub-command help")
    sub_parser_project = init_scaffold_project(subparsers)

    if len(sys.argv) == 1:
        # qrun
        parser.print_help()
        sys.exit(0)
    elif len(sys.argv) == 2:
        # print help for sub-commands
        if sys.argv[1] in ["-V", "--version"]:
            # qrun -V
            print(f"{__version__}")
        elif sys.argv[1] in ["-h", "--help"]:
            # qrun -h
            parser.print_help()
        elif sys.argv[1] == "create":
            # qrun startpro
            sub_parser_project.print_help()
        sys.exit(0)
    else:
        if sys.argv[1] == "create":
            args = parser.parse_args()
            main_scaffold_project(args)
        sys.exit(0)


if __name__ == "__main__":
    main()



from .. import Provider as GeoProvider


class Provider(GeoProvider):

    nationalities = (
        "Afegã",
        "Albanesa",
        "Arménia",
        "Angolana",
        "Argentina",
        "Austríaca",
        "Australiana",
        "Azerbaijã",
        "Belga",
        "Bulgara",
        "Boliviana",
        "Brasileira",
        "Bielorussa",
        "Canadiana",
        "Congolesa (República Democrática do Congo)",
        "Congolesa (República do Congo)",
        "Suíça",
        "Marfinense",
        "Chilena",
        "Chinesa",
        "Colombiana",
        "Costa-Riquenha",
        "Cubana",
        "Cabo-verdiana",
        "Cipriota",
        "Checa",
        "Alemã",
        "Dinamarquesa",
        "Dominicana",
        "Argelina",
        "Equatoriana",
        "Estónia",
        "Egípcia",
        "Espanhola",
        "Etíope",
        "Finlândesa",
        "Francesa",
        "Grega",
        "Guineense (Bissau)",
        "Croata",
        "Húngara",
        "Indonésia",
        "Irlandesa",
        "Israelita",
        "Indiana",
        "Iraquiana",
        "Iraniana",
        "Islandesa",
        "Italiana",
        "Jamaicana",
        "Japonesa",
        "Queniana",
        "Coreana",
        "Libanesa",
        "Lituana",
        "Luxemburguesa",
        "Letã",
        "Marroquina",
        "Moldava",
        "Birmanesa",
        "Maltesa",
        "Mexicana",
        "Moçambicana",
        "Nigeriana",
        "Holandesa",
        "Norueguesa",
        "Nepalesa",
        "Neozelandesa",
        "Peruana",
        "Filipina",
        "Paquistanesa",
        "Polaca",
        "Portuguesa",
        "Paraguaia",
        "Romena",
        "Russa",
        "Ruandesa",
        "Sudanesa",
        "Sueca",
        "Eslovena",
        "Eslovaca",
        "Senegalesa",
        "Somali",
        "Santomense",
        "Salvadorenha",
        "Tailandesa",
        "Tunisina",
        "Turca",
        "Ucraniana",
        "Britânica",
        "Americana",
        "Uruguaia",
        "Venezuelana",
        "Vietnamita",
        "Sul-Africana",
        "Sérvia",
        "Andorrenha",
        "Bósnia",
        "Camaronesa",
        "Georgiana",
        "Ganesa",
        "Gambiana",
        "Hondurenha",
        "Haitiana",
        "Cazaque",
        "Libanesa ",
        "Monegasca",
        "Maliana",
        "Mongol",
        "Mauritana",
        "Malaia",
        "Panamiana",
        "Saudita",
        "Singapurense",
        "Togolesa",
    )

    def nationality(self) -> str:
        """
        :example: 'Portuguesa'
        """
        return self.random_element(self.nationalities)



from .. import Provider as AddressProvider


class Provider(AddressProvider):
    building_number_formats = ("###", "##", "#")

    postcode_formats = ("#####",)

    city_formats = ("{{city_name}}",)

    street_name_formats = ("{{street_prefix}}{{street_suffix}}",)

    street_address_formats = ("{{street_name}} {{building_number}}",)

    address_formats = ("{{street_address}}\n{{postcode}} {{city}}",)

    # Data from:
    # https://www.avoindata.fi/data/en/dataset/kunnat/resource/b1cb9870-191f-4616-9c53-5388b7ca6beb
    cities = (
        "Alajärvi",
        "Alavieska",
        "Alavus",
        "Asikkala",
        "Askola",
        "Aura",
        "Akaa",
        "Brändö",
        "Eckerö",
        "Enonkoski",
        "Enontekiö",
        "Espoo",
        "Eura",
        "Eurajoki",
        "Evijärvi",
        "Finström",
        "Forssa",
        "Föglö",
        "Geta",
        "Haapajärvi",
        "Haapavesi",
        "Hailuoto",
        "Halsua",
        "Hamina",
        "Hammarland",
        "Hankasalmi",
        "Hanko",
        "Harjavalta",
        "Hartola",
        "Hattula",
        "Hausjärvi",
        "Heinävesi",
        "Helsinki",
        "Vantaa",
        "Hirvensalmi",
        "Hollola",
        "Honkajoki",
        "Huittinen",
        "Humppila",
        "Hyrynsalmi",
        "Hyvinkää",
        "Hämeenkyrö",
        "Hämeenlinna",
        "Heinola",
        "Ii",
        "Iisalmi",
        "Iitti",
        "Ikaalinen",
        "Ilmajoki",
        "Ilomantsi",
        "Inari",
        "Inkoo",
        "Isojoki",
        "Isokyrö",
        "Imatra",
        "Janakkala",
        "Joensuu",
        "Jokioinen",
        "Jomala",
        "Joroinen",
        "Joutsa",
        "Juuka",
        "Juupajoki",
        "Juva",
        "Jyväskylä",
        "Jämijärvi",
        "Jämsä",
        "Järvenpää",
        "Kaarina",
        "Kaavi",
        "Kajaani",
        "Kalajoki",
        "Kangasala",
        "Kangasniemi",
        "Kankaanpää",
        "Kannonkoski",
        "Kannus",
        "Karijoki",
        "Karkkila",
        "Karstula",
        "Karvia",
        "Kaskinen",
        "Kauhajoki",
        "Kauhava",
        "Kauniainen",
        "Kaustinen",
        "Keitele",
        "Kemi",
        "Keminmaa",
        "Kempele",
        "Kerava",
        "Keuruu",
        "Kihniö",
        "Kinnula",
        "Kirkkonummi",
        "Kitee",
        "Kittilä",
        "Kiuruvesi",
        "Kivijärvi",
        "Kokemäki",
        "Kokkola",
        "Kolari",
        "Konnevesi",
        "Kontiolahti",
        "Korsnäs",
        "Koski Tl",
        "Kotka",
        "Kouvola",
        "Kristiinankaupunki",
        "Kruunupyy",
        "Kuhmo",
        "Kuhmoinen",
        "Kumlinge",
        "Kuopio",
        "Kuortane",
        "Kurikka",
        "Kustavi",
        "Kuusamo",
        "Outokumpu",
        "Kyyjärvi",
        "Kärkölä",
        "Kärsämäki",
        "Kökar",
        "Kemijärvi",
        "Kemiönsaari",
        "Lahti",
        "Laihia",
        "Laitila",
        "Lapinlahti",
        "Lappajärvi",
        "Lappeenranta",
        "Lapinjärvi",
        "Lapua",
        "Laukaa",
        "Lemi",
        "Lemland",
        "Lempäälä",
        "Leppävirta",
        "Lestijärvi",
        "Lieksa",
        "Lieto",
        "Liminka",
        "Liperi",
        "Loimaa",
        "Loppi",
        "Loviisa",
        "Luhanka",
        "Lumijoki",
        "Lumparland",
        "Luoto",
        "Luumäki",
        "Lohja",
        "Parainen",
        "Maalahti",
        "Maarianhamina",
        "Marttila",
        "Masku",
        "Merijärvi",
        "Merikarvia",
        "Miehikkälä",
        "Mikkeli",
        "Muhos",
        "Multia",
        "Muonio",
        "Mustasaari",
        "Muurame",
        "Mynämäki",
        "Myrskylä",
        "Mäntsälä",
        "Mäntyharju",
        "Mänttä-Vilppula",
        "Naantali",
        "Nakkila",
        "Nivala",
        "Nokia",
        "Nousiainen",
        "Nurmes",
        "Nurmijärvi",
        "Närpiö",
        "Orimattila",
        "Oripää",
        "Orivesi",
        "Oulainen",
        "Oulu",
        "Padasjoki",
        "Paimio",
        "Paltamo",
        "Parikkala",
        "Parkano",
        "Pelkosenniemi",
        "Perho",
        "Pertunmaa",
        "Petäjävesi",
        "Pieksämäki",
        "Pielavesi",
        "Pietarsaari",
        "Pedersören kunta",
        "Pihtipudas",
        "Pirkkala",
        "Polvijärvi",
        "Pomarkku",
        "Pori",
        "Pornainen",
        "Posio",
        "Pudasjärvi",
        "Pukkila",
        "Punkalaidun",
        "Puolanka",
        "Puumala",
        "Pyhtää",
        "Pyhäjoki",
        "Pyhäjärvi",
        "Pyhäntä",
        "Pyhäranta",
        "Pälkäne",
        "Pöytyä",
        "Porvoo",
        "Raahe",
        "Raisio",
        "Rantasalmi",
        "Ranua",
        "Rauma",
        "Rautalampi",
        "Rautavaara",
        "Rautjärvi",
        "Reisjärvi",
        "Riihimäki",
        "Ristijärvi",
        "Rovaniemi",
        "Ruokolahti",
        "Ruovesi",
        "Rusko",
        "Rääkkylä",
        "Raasepori",
        "Saarijärvi",
        "Salla",
        "Salo",
        "Saltvik",
        "Sauvo",
        "Savitaipale",
        "Savonlinna",
        "Savukoski",
        "Seinäjoki",
        "Sievi",
        "Siikainen",
        "Siikajoki",
        "Siilinjärvi",
        "Simo",
        "Sipoo",
        "Siuntio",
        "Sodankylä",
        "Soini",
        "Somero",
        "Sonkajärvi",
        "Sotkamo",
        "Sottunga",
        "Sulkava",
        "Sund",
        "Suomussalmi",
        "Suonenjoki",
        "Sysmä",
        "Säkylä",
        "Vaala",
        "Sastamala",
        "Siikalatva",
        "Taipalsaari",
        "Taivalkoski",
        "Taivassalo",
        "Tammela",
        "Tampere",
        "Tervo",
        "Tervola",
        "Teuva",
        "Tohmajärvi",
        "Toholampi",
        "Toivakka",
        "Tornio",
        "Turku",
        "Pello",
        "Tuusniemi",
        "Tuusula",
        "Tyrnävä",
        "Ulvila",
        "Urjala",
        "Utajärvi",
        "Utsjoki",
        "Uurainen",
        "Uusikaarlepyy",
        "Uusikaupunki",
        "Vaasa",
        "Valkeakoski",
        "Valtimo",
        "Varkaus",
        "Vehmaa",
        "Vesanto",
        "Vesilahti",
        "Veteli",
        "Vieremä",
        "Vihti",
        "Viitasaari",
        "Vimpeli",
        "Virolahti",
        "Virrat",
        "Värdö",
        "Vöyri",
        "Ylitornio",
        "Ylivieska",
        "Ylöjärvi",
        "Ypäjä",
        "Ähtäri",
        "Äänekoski",
    )

    countries = (
        "Afganistan",
        "Alankomaat",
        "Albania",
        "Algeria",
        "Andorra",
        "Angola",
        "Antigua ja Barbuda",
        "Argentiina",
        "Armenia",
        "Australia",
        "Azerbaidžan",
        "Bahama",
        "Bahrain",
        "Bangladesh",
        "Barbados",
        "Belgia",
        "Belize",
        "Benin",
        "Bhutan",
        "Bolivia",
        "Bosnia ja Hertsegovina",
        "Botswana",
        "Brasilia",
        "Brunei",
        "Bulgaria",
        "Burkina",
        "Faso",
        "Burundi",
        "Chile",
        "Costa",
        "Rica",
        "Djibouti",
        "Dominica",
        "Dominikaaninen tasavalta",
        "Ecuador",
        "Egypti",
        "El",
        "Salvador",
        "Eritrea",
        "Espanja",
        "Etelä-Afrikka",
        "Korean tasavalta",
        "Etelä-Sudan",
        "Etiopia",
        "Fidži",
        "Filippiinit",
        "Gabon",
        "Gambia",
        "Georgia",
        "Ghana",
        "Grenada",
        "Guatemala",
        "Guinea-Bissau",
        "Guinea",
        "Guyana",
        "Haiti",
        "Honduras",
        "Indonesia",
        "Intia",
        "Irak",
        "Iran",
        "Irlanti",
        "Islanti",
        "Israel",
        "Italia",
        "Itä-Timor",
        "Itävalta",
        "Jamaika",
        "Japani",
        "Jemen",
        "Jordania",
        "Kambodža",
        "Kamerun",
        "Kanada",
        "Kap",
        "Verde",
        "Kazakstan",
        "Kenia",
        "Keski-Afrikan tasavalta",
        "Kiina",
        "Kirgisia",
        "Kiribati",
        "Kolumbia",
        "Komorit",
        "Kongon demokraattinen tasavalta",
        "Kongon tasavalta",
        "Kosovo",
        "Kreikka",
        "Kroatia",
        "Kuuba",
        "Kuwait",
        "Kypros",
        "Laos",
        "Latvia",
        "Lesotho",
        "Libanon",
        "Liberia",
        "Libya",
        "Liechtenstein",
        "Liettua",
        "Luxemburg",
        "Madagaskar",
        "Makedonia",
        "Malawi",
        "Malediivit",
        "Malesia",
        "Mali",
        "Malta",
        "Marokko",
        "Marshallinsaaret",
        "Mauritania",
        "Mauritius",
        "Meksiko",
        "Mikronesia",
        "Moldova",
        "Monaco",
        "Mongolia",
        "Montenegro",
        "Mosambik",
        "Myanmar",
        "Namibia",
        "Nauru",
        "Nepal",
        "Nicaragua",
        "Nigeria",
        "Niger",
        "Norja",
        "Norsunluurannikko",
        "Oman",
        "Pakistan",
        "Palau",
        "Panama",
        "Papua-Uusi-Guinea",
        "Paraguay",
        "Peru",
        "Korean demokraattinen kansantasavalta",
        "Portugali",
        "Puola",
        "Päiväntasaajan Guinea",
        "Qatar",
        "Ranska",
        "Romania",
        "Ruanda",
        "Ruotsi",
        "Saint Kitts ja Nevis",
        "Saint Lucia",
        "Saint Vincent ja Grenadiinit",
        "Saksa",
        "Salomonsaaret",
        "Sambia",
        "Samoa",
        "San Marino",
        "São Tomé ja Príncipe",
        "Saudi-Arabia",
        "Senegal",
        "Serbia",
        "Seychellit",
        "Sierra",
        "Leone",
        "Singapore",
        "Slovakia",
        "Slovenia",
        "Somalia",
        "Sri",
        "Lanka",
        "Sudan",
        "Suomi",
        "Suriname",
        "Swazimaa",
        "Sveitsi",
        "Syyria",
        "Tadžikistan",
        "Tansania",
        "Tanska",
        "Thaimaa",
        "Togo",
        "Tonga",
        "Trinidad ja Tobago",
        "Tšad",
        "Tšekki",
        "Tunisia",
        "Turkki",
        "Turkmenistan",
        "Tuvalu",
        "Uganda",
        "Ukraina",
        "Unkari",
        "Uruguay",
        "Uusi-Seelanti",
        "Uzbekistan",
        "Valko-Venäjä",
        "Vanuatu",
        "Vatikaanivaltio",
        "Venezuela",
        "Venäjä",
        "Vietnam",
        "Viro",
        "Yhdistyneet arabiemiirikunnat",
        "Yhdistynyt kuningaskunta",
        "Yhdysvallat",
        "Zimbabwe",
    )

    # Data from Finnish legislation:
    # https://www.finlex.fi/fi/laki/alkup/2019/20190978
    states = (
        "Ahvenanmaa",
        "Etelä-Karjala",
        "Etelä-Pohjanmaa",
        "Etelä-Savo",
        "Kainuu",
        "Kanta-Häme",
        "Keski-Pohjanmaa",
        "Keski-Suomi",
        "Kymenlaakso",
        "Lappi",
        "Pirkanmaa",
        "Pohjanmaa",
        "Pohjois-Karjala",
        "Pohjois-Pohjanmaa",
        "Pohjois-Savo",
        "Päijät-Häme",
        "Satakunta",
        "Uusimaa",
        "Varsinais-Suomi",
    )

    street_suffixes = ("tie", "katu", "polku", "kuja", "bulevardi")

    # Prefixes parsed from a street list of Helsinki:
    # http://kartta.hel.fi/ws/geoserver/avoindata/wfs?outputFormat=application/json&REQUEST=GetFeature&typeNames=avoindata:Helsinki_osoiteluettelo

    street_prefixes = (
        "Adolf Lindforsin ",
        "Agnes Sjöbergin ",
        "Agnetan",
        "Agricolan",
        "Ahomäen",
        "Ahvenkosken",
        "Aidasmäen",
        "Agroksen",
        "Agronomin",
        "Ahdekaunokin",
        "Bertel Jungin ",
        "Bertha Pauligin ",
        "Betlehemin",
        "Betoni",
        "Biologin",
        "Birger Kaipiaisen ",
        "Bysantin",
        "Böstaksen",
        "Bengalin",
        "Benktan",
        "Bergan",
        "Caloniuksen",
        "Capellan puisto",
        "Castrénin",
        "Chydeniuksen",
        "Cygnaeuksen",
        "Dagmarin",
        "Damaskuksen",
        "Degermosan",
        "Disan",
        "Dosentin",
        "Dunckerin",
        "Döbelnin",
        "Ehrensvärdin",
        "Eino Leinon ",
        "Elimäen",
        "Elisabeth Kochin ",
        "Eljaksen",
        "Elon",
        "Elon",
        "Edelfeltin",
        "Eduskunta",
        "Eerik Pyhän ",
        "Franzénin",
        "Fredrikin",
        "Freesen",
        "Fabianin",
        "Fagotti",
        "Fahlanderin puisto",
        "Fallin",
        "Fallkullan",
        "Fallpakan",
        "Fastbölen",
        "Gadolinin",
        "Gneissi",
        "Granfeltin",
        "Gunillan",
        "Gunnel Nymanin ",
        "Graniitti",
        "Gustav Pauligin ",
        "Gyldénin",
        "Gotlannin",
        "Haapa",
        "Haagan pappilan",
        "Haahka",
        "Haakoninlahden",
        "Haaksi",
        "Hankasuon",
        "Hannukselan",
        "Harakkamyllyn",
        "Harava",
        "Harbon",
        "Ilmattaren",
        "Ilomäen",
        "Ilotulitus",
        "Iltaruskon",
        "Iltatähden",
        "Ilves",
        "Immolan",
        "Ilkan",
        "Ida Ekmanin ",
        "Ies",
        "Jälsi",
        "Jämsän",
        "Jänkä",
        "Jänne",
        "Järkäle",
        "Jätkäsaaren",
        "Jättiläisen",
        "Jyvä",
        "Jägerhornin",
        "Jäkälä",
        "Kukkaniityn",
        "Kolsin",
        "Kolu",
        "Kolvi",
        "Kuhankeittäjän",
        "Katajaharjun",
        "Kiitäjän",
        "Kilpolan",
        "Kimalais",
        "Kimmon",
        "Laajasalon",
        "Laakavuoren",
        "Lemun",
        "Lentokapteenin ",
        "Lepolan",
        "Louhen",
        "Louhikko",
        "Lukkarimäen",
        "Laurinniityn",
        "Lautamiehen",
        "Mamsellimyllyn",
        "Mannerheimin",
        "Maanmittarin",
        "Maapadon",
        "Maa",
        "Maasalon",
        "Maasälvän",
        "Maatullin",
        "Malminkartanon",
        "Maneesi",
        "Niittylän",
        "Niemi",
        "Niitynperän",
        "Nikon",
        "Nils Westermarckin ",
        "Nordenskiöldin",
        "Nelikko",
        "Neon",
        "Nervanderin",
        "Neulapadon",
        "Ostos",
        "Orapihlaja",
        "Oras",
        "Orava",
        "Osmon",
        "Osuuskunnan",
        "Orisaaren",
        "Ormus",
        "Orvokki",
        "Oterman",
        "Pore",
        "Porin",
        "Porkkalan",
        "Pyörökiven",
        "Puusepän",
        "Puuska",
        "Pohjolan",
        "Poikasaarten",
        "Purjetuulen",
        "Puroniityn",
        "Rukkilan",
        "Ruko",
        "Rukoushuoneen",
        "Runebergin",
        "Runoilijan",
        "Runokylän",
        "Runonlaulajan",
        "Rantavaraston",
        "Rapakiven",
        "Rapolan",
        "Santerlan",
        "Saparon",
        "Sapilas",
        "Saramäen",
        "Saanatunturin",
        "Sade",
        "Sahaajan",
        "Salakka",
        "Salama",
        "Salava",
        "Tuomarinkylän",
        "Tuulilasin",
        "Taavetti Laitisen ",
        "Taavin",
        "Tahti",
        "Taimiston",
        "Tukkisillan",
        "Tuohikoivun",
        "Tyynelän",
        "Tyynylaavan",
        "Uussillan",
        "Urheilu",
        "Urkurin",
        "Urpu",
        "Uskalikon",
        "Usva",
        "Uudenkaupungin",
        "Uunilinnun",
        "Uunisepän",
        "Uurtajan",
        "Vanha Raja",
        "Veropellon",
        "Veräjämäen",
        "Vesakko",
        "Vesalan",
        "Vellikellon",
        "Verkko",
        "Verso",
        "Vaakalinnun",
        "Vaarna",
        "Wavulinin",
        "Walentin Chorellin ",
        "Wallinin",
        "Waseniuksen puisto",
        "Wecksellin",
        "Willebrandin",
        "Winqvistin",
        "Wäinö Aaltosen ",
        "Werner Wirénin ",
        "Yhteiskoulun",
        "Ylipalon",
        "Yllästunturin",
        "Ylä-Fallin ",
        "Yläkasken",
        "Ylänkö",
        "Ylätuvan",
        "Yrjö-Koskisen ",
        "Yrjön",
        "Yrttimaan",
        "Zaidan",
    )

    def street_prefix(self) -> str:
        return self.random_element(self.street_prefixes)

    def city_name(self) -> str:
        return self.random_element(self.cities)

    def administrative_unit(self) -> str:
        return self.random_element(self.states)

    state = administrative_unit



class QException(Exception):
    def __init__(self, msg=None, screen=None, stacktrace=None):
        self.msg = msg
        self.screen = screen
        self.stacktrace = stacktrace

    def __str__(self):
        _msg = f"Message: {self.msg}\n"
        if self.screen is not None:
            _msg += f"Screenshot: available via screen\n"
        if self.stacktrace is not None:
            stacktrace = "\n".join(self.stacktrace)
            _msg += f"Stacktrace:\n{stacktrace}"
        return _msg


class ElementTypeError(QException):
    """
    Find Element types Error
    """
    pass


class NoSuchElementException(QException):
    """
    Element could not found
    """
    pass


class TimeoutException(QException):
    """
    Thrown when a command does not complete in enough time.
    """
    pass



import os.path
import sys

api_run_content = """
import qrunner


if __name__ == '__main__':
    base_url = 'https://www.qizhidao.com'
    qrunner.main(
        case_path='test_dir',
        platform='api',
        base_url=base_url,
    )
"""

android_run_content = """
import qrunner


if __name__ == '__main__':
    qrunner.main(
        case_path = 'test_dir',
        platform='android',
        serial_no='UJK0220521066836',
        pkg_name='com.qizhidao.clientapp'
    )
"""

ios_run_content = """
import qrunner


if __name__ == '__main__':
    qrunner.main(
        case_path = 'test_dir',
        platform='ios',
        serial_no='00008101-000E646A3C29003A',
        pkg_name='com.qizhidao.company'
    )
"""

browser_run_content = """
import qrunner


if __name__ == '__main__':
    qrunner.main(
        case_path = 'test_dir',
        platform='browser',
        browser_name='chrome'
    )
"""

case_content_not_api = """
from qrunner import TestCase


class TestUiSample(TestCase):
    def test_go_my(self):
        self.click(resourceId='id/bottom_btn')
        self.click(resourceId='id/bottom_view', index=3)
        self.assertExist(text='我的订单')
        self.screenshot('test.png')
"""

case_content_api = """
from qrunner import TestCase, file_data


class TestApiSample(TestCase):

    @file_data('card_type', 'data.json')
    def test_getToolCardListForPc(self, card_type):
        path = '/api/qzd-bff-app/qzd/v1/home/getToolCardListForPc'
        pay_load = {"type": card_type}
        self.post(path, json=pay_load)
        self.assertCode(0)
"""

require_content = """qrunner
"""

ignore_content = "\n".join(
    ["__pycache__/*", "*.pyc", ".idea/*", ".DS_Store", "allure-results", "report"]
)

data_content = """
{
  "card_type": [0, 1, 2]
}
"""


def init_scaffold_project(subparsers):
    parser = subparsers.add_parser("create", help="Create a new project with template structure.")
    parser.add_argument('-n', '--name', dest='name', type=str, default='my_pro', help='Specify new project name.')
    parser.add_argument('-t', '--platform', dest='platform', type=str, default='api', help='测试平台: api、android、ios、browser')
    return parser


def create_scaffold(project_name, platform):
    """ create scaffold with specified project name.
    """

    def create_folder(path):
        os.makedirs(path)
        msg = f"created folder: {path}"
        print(msg)

    def create_file(path, file_content=""):
        with open(path, "w", encoding="utf-8") as f:
            f.write(file_content)
        msg = f"created file: {path}"
        print(msg)

    create_folder(project_name)
    create_file(
        os.path.join(project_name, ".gitignore"),
        ignore_content,
    )
    create_file(
        os.path.join(project_name, "requirements.txt"),
        require_content,
    )
    create_folder(os.path.join(project_name, 'test_dir'))
    create_file(
        os.path.join(project_name, 'test_dir', "__init__.py"),
        '',
    )
    if platform == 'api':
        create_file(
            os.path.join(project_name, "run.py"),
            api_run_content,
        )
        create_file(
            os.path.join(project_name, 'test_dir', "test_api_sample.py"),
            case_content_api,
        )
    elif platform == 'android':
        create_file(
            os.path.join(project_name, "run.py"),
            android_run_content,
        )
        create_file(
            os.path.join(project_name, 'test_dir', "test_api_sample.py"),
            case_content_not_api,
        )
    elif platform == 'ios':
        create_file(
            os.path.join(project_name, "run.py"),
            ios_run_content,
        )
        create_file(
            os.path.join(project_name, 'test_dir', "test_api_sample.py"),
            case_content_not_api,
        )
    elif platform == 'browser':
        create_file(
            os.path.join(project_name, "run.py"),
            browser_run_content,
        )
        create_file(
            os.path.join(project_name, 'test_dir', "test_api_sample.py"),
            case_content_not_api,
        )
    else:
        print(f'不支持的平台: {platform}')
        sys.exit()
    create_folder(os.path.join(project_name, 'test_data'))
    create_file(
        os.path.join(project_name, 'test_data', "data.json"),
        data_content,
    )
    create_folder(os.path.join(project_name, 'report'))
    return 0


def main_scaffold_project(args):
    sys.exit(create_scaffold(args.name, args.platform))




from ..ar_AA import Provider as ArabicDateTimeProvider


class Provider(ArabicDateTimeProvider):
    MONTH_NAMES = {
        "01": "يناير",
        "02": "فبراير",
        "03": "مارس",
        "04": "أبريل",
        "05": "مايو",
        "06": "يونيو",
        "07": "يوليو",
        "08": "أغسطس",
        "09": "سبتمبر",
        "10": "أكتوبر",
        "11": "نوفمبر",
        "12": "ديسمبر",
    }



from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{first_name}} {{last_name}} s.p.",
    )

    company_suffixes = (
        "d.o.o.",
        "d.d.",
    )



from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = [
        "{{last_name}} {{company_suffix}}",
        "{{last_name}} {{company_suffix}}",
        "{{last_name}} {{company_suffix}}",
        "{{last_name}}-{{last_name}} {{company_suffix}}",
        "{{last_name}}, {{last_name}} og {{last_name}}",
        "{{last_name}}-{{last_name}}",
    ]

    company_suffixes = [
        "Gruppen",
        "AS",
        "ASA",
        "BA",
        "RFH",
        "og Sønner",
        "& co.",
    ]



from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``az_AZ`` locale.

    Word list is based on the source(s) below with some filtering.

    Sources:

    - https://1000mostcommonwords.com/1000-most-common-azerbaijani-words/
    """

    word_list = (
        "kimi",
        "mən",
        "olmaq",
        "at",
        "bir",
        "var",
        "bu",
        "dən",
        "tərəfindən",
        "isti",
        "bilərsiniz",
        "həyata",
        "digər",
        "etmək",
        "onların",
        "vaxt",
        "əgər",
        "olacaq",
        "necə",
        "bildirib",
        "bir",
        "hər",
        "demək",
        "yoxdur",
        "dəst",
        "üç",
        "istəyirəm",
        "hava",
        "quyu",
        "oynamaq",
        "kiçik",
        "son",
        "qoymaq",
        "ev",
        "oxumaq",
        "əl",
        "port",
        "böyük",
        "sehr",
        "əlavə",
        "etmək",
        "hətta",
        "torpaq",
        "burada",
        "lazımdır",
        "böyük",
        "yüksək",
        "belə",
        "izləmək",
        "akt",
        "niyə",
        "soruşmaq",
        "oxumaq",
        "dəyişiklik",
        "getdi",
        "yüngül",
        "cür",
        "müstəqil",
        "ehtiyac",
        "ev",
        "şəkil",
        "çalışmaq",
        "azad",
        "yenidən",
        "heyvan",
        "nöqtə",
        "ana",
        "dünya",
        "yaxın",
        "qurmaq",
        "özü",
        "torpaq",
        "ata",
        "hər",
        "hansı",
        "bir",
        "yeni",
        "iş",
        "hissə",
        "almaq",
        "yer",
        "etdi",
        "yaşamaq",
        "harada",
        "sonra",
        "cümlə",
        "böyük",
    )



import time
from jira import JIRA


def create_issue(title: str, content: str, to='kun.chen', env='pre', epc_id='APP-23662'):
    """
    :param title: bug标题
    :param content: bug详细描述
    :param to: 指派给人员的姓名，如kang.yang
    :param env: 问题环境，如nts、test、pre、prd
    :param epc_id: epic页面链接最后的id，如APP-23662
    :return: {
        'title': 'bug标题',
        'issue_url': 'https://jira.qizhidao.com/browse/APP-38532'
    }
    """

    jira = JIRA(server='http://jira.qizhidao.com', auth=('kang.yang', 'wz888888'))
    # print(jira.projects())

    if env == 'test':
        env_name = 'SIT环境'
    elif env == 'pre':
        env_name = 'PRE环境'
    elif env == 'nts':
        env_name = 'NTS环境'
    elif env == 'prd':
        env_name = '线上环境'
    else:
        env_name = 'PRE环境'

    date_now = time.strftime('%Y-%m-%d')

    issue_dict = {
        'project': '10002',  # 企知道APP，设为默认
        'summary': title,
        'description': content,
        'priority': {'name': 'Medium'},
        'assignee': {'name': to},
        'customfield_10612': {'value': env_name},
        'customfield_10400': {'value': '一般'},
        'duedate': date_now,
        'issuetype': {'name': 'Bug'},
        'customfield_10000': epc_id
    }

    new_issue = jira.create_issue(issue_dict)

    issue_info = {
        'title': title,
        'issue_url': f"https://jira.qizhidao.com/browse/{new_issue.raw.get('key')}"
    }
    print(issue_info)
    return issue_info


# 批量提单
def create_issues(info_list: list):
    """
    param info_list: [
        {'title': 'bug标题', 'content': 'bug描述'},
        ...
    ]
    return: [
        {'title': 'bug标题', 'issue_url': 'bug链接'},
        ...
    ]
    """

    result_list = []
    for info in info_list:
        title = info.get('title')
        content = info.get('content')
        issue_info = create_issue(title, content)
        result_list.append(issue_info)
    return result_list


if __name__ == '__main__':
    create_issue('test', 'test')



from .. import Provider as AddressProvider


class Provider(AddressProvider):
    # City and States names taken from wikipedia
    # Street format taken from some common famous places in India
    # Link for cities: https://en.wikipedia.org/wiki/List_of_cities_in_India_by_population
    # Link for States: https://en.wikipedia.org/wiki/States_and_union_territories_of_India
    # Links for street name formats: https://www.mumbai77.com/city/3313/travel/old-new-street-names/

    city_formats = ("{{city_name}}",)

    street_name_formats = (
        "{{last_name}} Nagar",
        "{{last_name}} Zila",
        "{{last_name}} Street",
        "{{last_name}} Ganj",
        "{{last_name}} Road",
        "{{last_name}} Path",
        "{{last_name}} Marg",
        "{{last_name}} Chowk",
        "{{last_name}} Circle",
        "{{last_name}}",
    )

    street_address_formats = (
        "{{building_number}}, {{street_name}}",
        "{{building_number}}\n{{street_name}}",
    )

    address_formats = (
        "{{street_address}}\n{{city}} {{postcode}}",
        "{{street_address}}\n{{city}}-{{postcode}}",
        "{{street_address}}, {{city}} {{postcode}}",
        "{{street_address}}, {{city}}-{{postcode}}",
    )

    building_number_formats = ("H.No. ###", "###", "H.No. ##", "##", "##/##", "##/###")

    postcode_formats = ("######",)

    cities = (
        "Mumbai",
        "Delhi",
        "Kolkata",
        "Chennai",
        "Bangalore",
        "Hyderabad",
        "Ahmedabad",
        "Kanpur",
        "Pune",
        "Surat",
        "Jaipur",
        "Lucknow",
        "Nagpur",
        "Indore",
        "Bhopal",
        "Ludhiana",
        "Patna",
        "Visakhapatnam",
        "Vadodara",
        "Agra",
        "Thane",
        "Kalyan-Dombivli",
        "Varanasi",
        "Ranchi",
        "Nashik",
        "Dhanbad",
        "Faridabad",
        "Meerut",
        "Pimpri-Chinchwad",
        "Howrah",
        "Allahabad",
        "Ghaziabad",
        "Rajkot",
        "Amritsar",
        "Jabalpur",
        "Coimbatore",
        "Madurai",
        "Srinagar",
        "Aurangabad",
        "Solapur",
        "Vijayawada",
        "Jodhpur",
        "Gwalior",
        "Guwahati",
        "Chandigarh",
        "Hubli–Dharwad",
        "Mysore",
        "Tiruchirappalli",
        "Bareilly",
        "Jalandhar",
        "Navi Mumbai",
        "Salem",
        "Kota",
        "Vasai-Virar",
        "Aligarh",
        "Moradabad",
        "Bhubaneswar",
        "Gorakhpur",
        "Raipur",
        "Bhiwandi",
        "Kochi",
        "Jamshedpur",
        "Bhilai",
        "Amravati",
        "Cuttack",
        "Warangal",
        "Bikaner",
        "Mira-Bhayandar",
        "Guntur",
        "Bhavnagar",
        "Durgapur",
        "Kolhapur",
        "Ajmer",
        "Asansol",
        "Ulhasnagar",
        "Siliguri",
        "Jalgaon",
        "Saharanpur",
        "Jamnagar",
        "Bhatpara",
        "Sangli-Miraj & Kupwad",
        "Kozhikode",
        "Nanded",
        "Ujjain",
        "Dehradun",
        "Rourkela",
        "Gulbarga",
        "Tirunelveli",
        "Malegaon",
        "Akola",
        "Belgaum",
        "Mangalore",
        "Bokaro",
        "South Dumdum",
        "Udaipur",
        "Gaya",
        "Maheshtala",
        "Jhansi",
        "Nellore",
        "Jammu",
        "Thiruvananthapuram",
        "Davanagere",
        "Kollam",
        "Panihati",
        "Kurnool",
        "Tiruppur",
        "Dhule",
        "Bhagalpur",
        "Rajpur Sonarpur",
        "Kakinada",
        "Thrissur",
        "Bellary",
        "Muzaffarnagar",
        "Korba",
        "Rajahmundry",
        "Kamarhati",
        "Ambattur",
        "Berhampur",
        "Ahmednagar",
        "Muzaffarpur",
        "Noida",
        "Patiala",
        "Mathura",
        "New Delhi",
        "Latur",
        "Sambalpur",
        "Shahjahanpur",
        "Kulti",
        "Chandrapur",
        "Nizamabad",
        "Rohtak",
        "Bardhaman",
        "Rampur",
        "Bhilwara",
        "Firozabad",
        "Bilaspur",
        "Shimoga",
        "Agartala",
        "Gopalpur",
        "Darbhanga",
        "Panipat",
        "Bally",
        "Alwar",
        "Parbhani",
        "Ichalkaranji",
        "Anantapuram",
        "Baranagar",
        "Tumkur",
        "Ramagundam",
        "Jalna",
        "Durg",
        "Sagar",
        "Bihar Sharif",
        "Dewas",
        "Barasat",
        "Avadi",
        "Farrukhabad",
        "Aizawl",
        "Tirupati",
        "Bijapur",
        "Satara",
        "Satna",
        "Ratlam",
        "Imphal",
        "Pondicherry",
        "North Dumdum",
        "Anantapur",
        "Khammam",
        "Ozhukarai",
        "Bathinda",
        "Thoothukudi",
        "Thanjavur",
        "Naihati",
        "Sonipat",
        "Mau",
        "Tiruvottiyur",
        "Hapur",
        "Sri Ganganagar",
        "Karnal",
        "Etawah",
        "Nagercoil",
        "Raichur",
        "Raurkela Industrial Township",
        "Secunderabad",
        "Karimnagar",
        "Mirzapur",
        "Bharatpur",
        "Ambarnath",
        "Arrah",
        "Uluberia",
        "Serampore",
        "Dindigul",
        "Gandhinagar",
        "Burhanpur",
        "Nadiad",
        "Eluru",
        "Yamunanagar",
        "Kharagpur",
        "Munger",
        "Pali",
        "Katni",
        "Singrauli",
        "Tenali",
        "Sikar",
        "Silchar",
        "Rewa",
        "Sambhal",
        "Machilipatnam",
        "Vellore",
        "Alappuzha",
        "Bulandshahr",
        "Haridwar",
        "Vijayanagaram",
        "Erode",
        "Gurgaon",
        "Bidar",
        "Bhusawal",
        "Khandwa",
        "Purnia",
        "Haldia",
        "Chinsurah",
        "Bhiwani",
        "Raebareli",
        "Junagadh",
        "Bahraich",
        "Gandhidham",
        "Mango",
        "Raiganj",
        "Amroha",
        "Sultan Pur Majra",
        "Hospet",
        "Bidhannagar",
        "Malda",
        "Sirsa",
        "Berhampore",
        "Jaunpur",
        "Surendranagar Dudhrej",
        "Madhyamgram",
        "Kirari Suleman Nagar",
        "Bhind",
        "Nandyal",
        "Chittoor",
        "Bhalswa Jahangir Pur",
        "Fatehpur",
        "Morena",
        "Nangloi Jat",
        "Ongole",
        "Karawal Nagar",
        "Shivpuri",
        "Morbi",
        "Unnao",
        "Pallavaram",
        "Kumbakonam",
        "Shimla",
        "Mehsana",
        "Panchkula",
        "Orai",
        "Ambala",
        "Dibrugarh",
        "Guna",
        "Danapur",
        "Sasaram",
        "Anand",
        "Kottayam",
        "Hazaribagh",
        "Kadapa",
        "Saharsa",
        "Nagaon",
        "Loni",
        "Hajipur",
        "Dehri",
        "Bettiah",
        "Katihar",
        "Deoghar",
        "Jorhat",
        "Siwan",
        "Panvel",
        "Hosur",
        "Tinsukia",
        "Bongaigaon",
        "Motihari",
        "Jamalpur",
        "Suryapet",
        "Begusarai",
        "Miryalaguda",
        "Proddatur",
        "Karaikudi",
        "Kishanganj",
        "Phusro",
        "Buxar",
        "Tezpur",
        "Jehanabad",
        "Aurangabad",
        "Chapra",
        "Ramgarh",
        "Gangtok",
        "Adoni",
        "Amaravati",
        "Ballia",
        "Bhimavaram",
        "Dharmavaram",
        "Giridih",
        "Gudivada",
        "Guntakal",
        "Hindupur",
        "Kavali",
        "Khora ",
        "Ghaziabad",
        "Madanapalle",
        "Mahbubnagar",
        "Medininagar",
        "Narasaraopet",
        "Phagwara",
        "Pudukkottai",
        "Srikakulam",
        "Tadepalligudem",
        "Tadipatri",
        "Udupi",
    )

    states = (
        "Andhra Pradesh",
        "Arunachal Pradesh",
        "Assam",
        "Bihar",
        "Chhattisgarh",
        "Goa",
        "Gujarat",
        "Haryana",
        "Himachal Pradesh",
        "Jharkhand",
        "Karnataka",
        "Kerala",
        "Madhya Pradesh",
        "Maharashtra",
        "Manipur",
        "Meghalaya",
        "Mizoram",
        "Nagaland",
        "Odisha",
        "Punjab",
        "Rajasthan",
        "Sikkim",
        "Tamil Nadu",
        "Telangana",
        "Tripura",
        "Uttar Pradesh",
        "Uttarakhand",
        "West Bengal",
    )

    def city_name(self) -> str:
        return self.random_element(self.cities)

    def administrative_unit(self) -> str:
        return self.random_element(self.states)

    state = administrative_unit
